@@ -8,9 +8,9 @@
  *
  * Purpose: Header file for fwknop client test program.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -34,9 +34,4 @@
 
 #include "fwknop_common.h"
 
-/* Used by the get_user_pw function below.
-*/
-#define CRYPT_OP_ENCRYPT 1
-#define CRYPT_OP_DECRYPT 2
-
 #endif  /* FWKNOP_H */
