@@ -7,9 +7,9 @@
  *
  * Purpose: Header file for fwknop config_init.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@
 #define FWKNOP_COMMON_H
 
 #include "common.h"
+#include "log_msg.h"
 
 /* My Name and Version
 */
@@ -55,7 +56,7 @@
 #define TIME_OFFSET_DAYS        86400
 
 /* For resolving the allow IP via HTTP and sending SPA packets over
- * HTTP -  http://www.whatismyip.com/automation/n09230945.asp 
+ * HTTP -  http://www.whatismyip.com/automation/n09230945.asp
     #define HTTP_RESOLVE_HOST          "www.whatismyip.com"
     #define HTTP_RESOLVE_URL           "/automation/n09230945.asp"
   * --DSS Note: The whatismyip.com site has some usage restrictions.
@@ -71,6 +72,8 @@
 #define MAX_HOSTNAME_LEN            70
 #define MAX_URL_HOST_LEN            256
 #define MAX_URL_PATH_LEN            1024
+#define MAX_KEY_LEN                 128
+#define MAX_B64_KEY_LEN             180
 
 /* fwknop client configuration parameters and values
 */
@@ -78,13 +81,18 @@ typedef struct fko_cli_options
 {
     char config_file[MAX_PATH_LEN];
     char access_str[MAX_PATH_LEN];
+    char rc_file[MAX_PATH_LEN];
+    char key_gen_file[MAX_PATH_LEN];
     char server_command[MAX_LINE_LEN];
     char get_key_file[MAX_PATH_LEN];
+    char get_hmac_key_file[MAX_PATH_LEN];
     char save_packet_file[MAX_PATH_LEN];
     int  save_packet_file_append;
     int  show_last_command;
     int  run_last_command;
+    char args_save_file[MAX_PATH_LEN];
     int  no_save_args;
+    int  use_hmac;
     char spa_server_str[MAX_SERVER_STR_LEN];  /* may be a hostname */
     char allow_ip_str[MAX_IPV4_STR_LEN];
     char spoof_ip_src_str[MAX_IPV4_STR_LEN];
@@ -94,6 +102,20 @@ typedef struct fko_cli_options
     char gpg_signer_key[MAX_GPG_KEY_ID];
     char gpg_home_dir[MAX_PATH_LEN];
 
+    /* Encryption keys read from a .fwknoprc stanza
+    */
+    char key[MAX_KEY_LEN+1];
+    char key_base64[MAX_B64_KEY_LEN+1];
+    int  key_len;
+    char hmac_key[MAX_KEY_LEN+1];
+    char hmac_key_base64[MAX_B64_KEY_LEN+1];
+    int  hmac_key_len;
+    int  have_key;
+    int  have_base64_key;
+    int  have_hmac_key;
+    int  have_hmac_base64_key;
+    int  hmac_type;
+
     /* NAT access
     */
     char nat_access_str[MAX_PATH_LEN];
@@ -117,23 +139,31 @@ typedef struct fko_cli_options
     unsigned int spa_dst_port;
     unsigned int spa_src_port; /* only used with --source-port */
 
+    short digest_type;
+    int encryption_mode;
+
     int spa_icmp_type;  /* only used in '-P icmp' mode */
     int spa_icmp_code;  /* only used in '-P icmp' mode */
 
-    unsigned int digest_type;
-
     /* Various command-line flags */
     unsigned char   verbose; /* --verbose mode */
     unsigned char   version; /* --version */
     unsigned char   test;
     unsigned char   use_gpg;
     unsigned char   use_gpg_agent;
+    unsigned char   gpg_no_signing_pw;
     int             time_offset_plus;
     int             time_offset_minus;
     int             fw_timeout;
+    int             key_gen;
 
     char            use_rc_stanza[MAX_LINE_LEN];
     unsigned char   got_named_stanza;
+    unsigned char   save_rc_stanza;
+    unsigned char   force_save_rc_stanza;
+    unsigned char   stanza_list;
+
+    int input_fd;
 
     //char            config_file[MAX_PATH_LEN];
 
