@@ -7,9 +7,9 @@
  *
  * Purpose: Header file for getpasswd.c.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -33,7 +33,11 @@
 
 /* Prototypes
 */
-char* getpasswd(const char *prompt);
-char* getpasswd_file(fko_ctx_t ctx, const fko_cli_options_t *options);
+char* getpasswd(const char *prompt, int fd);
+
+/* This can be used to acquire an encryption key or HMAC key
+*/
+int get_key_file(char *key, int *key_len, const char *key_file,
+    fko_ctx_t ctx, const fko_cli_options_t *options);
 
 #endif  /* GETPASSWD_H */
