@@ -8,9 +8,9 @@
  *
  * Purpose: Network-related functions for the fwknop client
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -32,36 +32,24 @@
 #include "spa_comm.h"
 #include "utils.h"
 
-static void
-print_proto(const int proto)
-{
-    switch (proto) {
-        case FKO_PROTO_UDP:
-            printf("udp");
-            break;
-        case FKO_PROTO_TCP_RAW:
-            printf("tcpraw");
-            break;
-        case FKO_PROTO_TCP:
-            printf("tcp");
-            break;
-        case FKO_PROTO_ICMP:
-            printf("icmp");
-            break;
-        case FKO_PROTO_HTTP:
-            printf("http");
-            break;
-    }
-    return;
-}
-
 static void
 dump_transmit_options(const fko_cli_options_t *options)
 {
-    printf("Generating SPA packet:\n    protocol: ");
-    print_proto(options->spa_proto),
-    printf("\n        port: %d\n", options->spa_dst_port);
-    printf("     IP/host: %s\n", options->spa_server_str);
+    char proto_str[PROTOCOL_BUFSIZE] = {0};   /* Protocol string */
+
+    proto_inttostr(options->spa_proto, proto_str, sizeof(proto_str));
+
+    log_msg(LOG_VERBOSITY_INFO, "Generating SPA packet:");
+    log_msg(LOG_VERBOSITY_INFO, "            protocol: %s", proto_str);
+
+    if (options->spa_src_port)
+        log_msg(LOG_VERBOSITY_INFO, "         source port: %d", options->spa_src_port);
+    else
+        log_msg(LOG_VERBOSITY_INFO, "         source port: <OS assigned>");
+
+    log_msg(LOG_VERBOSITY_INFO, "    destination port: %d", options->spa_dst_port);
+    log_msg(LOG_VERBOSITY_INFO, "             IP/host: %s", options->spa_server_str);
+
     return;
 }
 
@@ -93,23 +81,6 @@ chksum(unsigned short *buf, int nbytes)
     return (unsigned short) ~sum;
 }
 
-/*
-static int is_ip(char *str)
-{
-    int rv = 1;
-    unsigned int i;
-
-    for (i=0; i < strlen(str); i++) {
-        if (str[i] != '.' && ! isdigit(str[i])) {
-            rv = 0;
-            break;
-        }
-    }
-
-    return rv;
-}
-*/
-
 /* Send the SPA data via UDP packet.
 */
 static int
@@ -118,12 +89,12 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 {
     int     sock, res=0, error;
     struct  addrinfo *result, *rp, hints;
-    char    port_str[MAX_PORT_STR_LEN+1];
+    char    port_str[MAX_PORT_STR_LEN+1] = {0};
 
     if (options->test)
     {
-        fprintf(stderr,
-            "test mode enabled, SPA packet not actually sent.\n");
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "test mode enabled, SPA packet not actually sent.");
         return res;
     }
 
@@ -153,8 +124,8 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 
     if (error != 0)
     {
-        fprintf(stderr, "error in getaddrinfo: %s\n", gai_strerror(error));
-        exit(EXIT_FAILURE);
+        log_msg(LOG_VERBOSITY_ERROR, "error in getaddrinfo: %s", gai_strerror(error));
+        return -1;
     }
 
     for (rp = result; rp != NULL; rp = rp->ai_next) {
@@ -174,8 +145,10 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
     }
 
     if (rp == NULL) {
-        perror("send_spa_packet_tcp_or_udp: Could not create socket: ");
-        exit(EXIT_FAILURE);
+        log_msg(LOG_VERBOSITY_ERROR,
+                "send_spa_packet_tcp_or_udp: Could not create socket: ",
+                strerror(errno));
+        return -1;
     }
 
     freeaddrinfo(result);
@@ -184,12 +157,12 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 
     if(res < 0)
     {
-        perror("send_spa_packet_tcp_or_udp: write error: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_tcp_or_udp: write error: ", strerror(errno));
     }
     else if(res != sd_len)
     {
-        fprintf(stderr,
-            "[#] Warning: bytes sent (%i) not spa data length (%i).\n",
+        log_msg(LOG_VERBOSITY_WARNING,
+            "[#] Warning: bytes sent (%i) not spa data length (%i).",
             res, sd_len
         );
     }
@@ -211,8 +184,8 @@ send_spa_packet_tcp_raw(const char *spa_data, const int sd_len,
     const fko_cli_options_t *options)
 {
 #ifdef WIN32
-    fprintf(stderr,
-        "send_spa_packet_tcp_raw: raw packets are not yet supported.\n");
+    log_msg(LOG_VERBOSITY_ERROR,
+        "send_spa_packet_tcp_raw: raw packets are not yet supported.");
     return(-1);
 #else
     int  sock, res = 0;
@@ -230,15 +203,15 @@ send_spa_packet_tcp_raw(const char *spa_data, const int sd_len,
 
     if (options->test)
     {
-        fprintf(stderr,
-            "test mode enabled, SPA packet not actually sent.\n");
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "test mode enabled, SPA packet not actually sent.");
         return res;
     }
 
     sock = socket (PF_INET, SOCK_RAW, IPPROTO_RAW);
     if (sock < 0)
     {
-        perror("send_spa_packet_tcp_raw: create socket: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_tcp_raw: create socket: ", strerror(errno));
         return(sock);
     }
 
@@ -292,19 +265,19 @@ send_spa_packet_tcp_raw(const char *spa_data, const int sd_len,
      * doesn't try to insert its own header into the packet.
     */
     if (setsockopt (sock, IPPROTO_IP, IP_HDRINCL, so_val, sizeof(one)) < 0)
-        perror("send_spa_packet_tcp_raw: setsockopt HDRINCL: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_tcp_raw: setsockopt HDRINCL: ", strerror(errno));
 
     res = sendto (sock, pkt_data, iph->tot_len, 0,
         (struct sockaddr *)daddr, sizeof(*daddr));
 
     if(res < 0)
     {
-        perror("send_spa_packet_tcp_raw: sendto error: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_tcp_raw: sendto error: ", strerror(errno));
     }
     else if(res != sd_len + hdrlen) /* account for the header ?*/
     {
-        fprintf(stderr,
-            "[#] Warning: bytes sent (%i) not spa data length (%i).\n",
+        log_msg(LOG_VERBOSITY_WARNING,
+            "[#] Warning: bytes sent (%i) not spa data length (%i).",
             res, sd_len
         );
     }
@@ -324,8 +297,8 @@ send_spa_packet_udp_raw(const char *spa_data, const int sd_len,
     const fko_cli_options_t *options)
 {
 #ifdef WIN32
-    fprintf(stderr,
-        "send_spa_packet_udp_raw: raw packets are not yet supported.\n");
+    log_msg(LOG_VERBOSITY_ERROR,
+        "send_spa_packet_udp_raw: raw packets are not yet supported.");
     return(-1);
 #else
     int  sock, res = 0;
@@ -343,15 +316,15 @@ send_spa_packet_udp_raw(const char *spa_data, const int sd_len,
 
     if (options->test)
     {
-        fprintf(stderr,
-            "test mode enabled, SPA packet not actually sent.\n");
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "test mode enabled, SPA packet not actually sent.");
         return res;
     }
 
     sock = socket (PF_INET, SOCK_RAW, IPPROTO_RAW);
     if (sock < 0)
     {
-        perror("send_spa_packet_udp_raw: create socket: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_udp_raw: create socket: ", strerror(errno));
         return(sock);
     }
 
@@ -391,19 +364,19 @@ send_spa_packet_udp_raw(const char *spa_data, const int sd_len,
      * doesn't try to insert its own header into the packet.
     */
     if (setsockopt (sock, IPPROTO_IP, IP_HDRINCL, so_val, sizeof(one)) < 0)
-        perror("send_spa_packet_udp_raw: setsockopt HDRINCL: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_udp_raw: setsockopt HDRINCL: ", strerror(errno));
 
     res = sendto (sock, pkt_data, iph->tot_len, 0,
         (struct sockaddr *)daddr, sizeof(*daddr));
 
     if(res < 0)
     {
-        perror("send_spa_packet_udp_raw: sendto error: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_udp_raw: sendto error: ", strerror(errno));
     }
     else if(res != sd_len + hdrlen) /* account for the header ?*/
     {
-        fprintf(stderr,
-            "[#] Warning: bytes sent (%i) not spa data length (%i).\n",
+        log_msg(LOG_VERBOSITY_WARNING,
+            "[#] Warning: bytes sent (%i) not spa data length (%i).",
             res, sd_len
         );
     }
@@ -423,7 +396,7 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
     const fko_cli_options_t *options)
 {
 #ifdef WIN32
-    fprintf(stderr, "send_spa_packet_icmp: raw packets are not yet supported.\n");
+    log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_icmp: raw packets are not yet supported.");
     return(-1);
 #else
     int res = 0, sock;
@@ -441,8 +414,8 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
 
     if (options->test)
     {
-        fprintf(stderr,
-            "test mode enabled, SPA packet not actually sent.\n");
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "test mode enabled, SPA packet not actually sent.");
         return res;
     }
 
@@ -450,7 +423,7 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
 
     if (sock < 0)
     {
-        perror("send_spa_packet_icmp: create socket: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_icmp: create socket: ", strerror(errno));
         return(sock);
     }
 
@@ -496,18 +469,18 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
      * doesn't try to insert its own header into the packet.
     */
     if (setsockopt (sock, IPPROTO_IP, IP_HDRINCL, so_val, sizeof(one)) < 0)
-        perror("send_spa_packet_icmp: setsockopt HDRINCL: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_icmp: setsockopt HDRINCL: ", strerror(errno));
 
     res = sendto (sock, pkt_data, iph->tot_len, 0,
         (struct sockaddr *)daddr, sizeof(*daddr));
 
     if(res < 0)
     {
-        perror("send_spa_packet_icmp: sendto error: ");
+        log_msg(LOG_VERBOSITY_ERROR, "send_spa_packet_icmp: sendto error: ", strerror(errno));
     }
     else if(res != sd_len + hdrlen) /* account for icmp header */
     {
-        fprintf(stderr, "[#] Warning: bytes sent (%i) not spa data length (%i).\n",
+        log_msg(LOG_VERBOSITY_WARNING, "[#] Warning: bytes sent (%i) not spa data length (%i).",
             res, sd_len);
     }
 
@@ -524,14 +497,15 @@ static int
 send_spa_packet_http(const char *spa_data, const int sd_len,
     fko_cli_options_t *options)
 {
-    char http_buf[HTTP_MAX_REQUEST_LEN], *spa_data_copy = NULL;
+    char http_buf[HTTP_MAX_REQUEST_LEN] = {0}, *spa_data_copy = NULL;
     char *ndx = options->http_proxy;
-    int  i, proxy_port = 0;
+    int  i, proxy_port = 0, is_err;
 
     spa_data_copy = malloc(sd_len+1);
     if (spa_data_copy == NULL)
     {
-        exit(EXIT_FAILURE);
+        log_msg(LOG_VERBOSITY_ERROR, "[*] Fatal, could not allocate memory.");
+        return -1;
     }
     memcpy(spa_data_copy, spa_data, sd_len+1);
 
@@ -567,14 +541,22 @@ send_spa_packet_http(const char *spa_data, const int sd_len,
             memmove(ndx, ndx+7, strlen(ndx)+1);
 
         /* If there is a colon assume the proxy hostame or IP is on the left
-         * and the proxy port is on the right. So we make the : a \0 and 
+         * and the proxy port is on the right. So we make the : a \0 and
          * extract the port value.
         */
         ndx = strchr(options->http_proxy, ':');
         if(ndx)
         {
             *ndx = '\0';
-            proxy_port = atoi(ndx+1);
+            proxy_port = strtol_wrapper(ndx+1, 1, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+            if(is_err != FKO_SUCCESS)
+            {
+                log_msg(LOG_VERBOSITY_ERROR,
+                    "[-] proxy port value is invalid, must be in [%d-%d]",
+                    1, MAX_PORT);
+                free(spa_data_copy);
+                return -1;
+            }
         }
 
         /* If we have a valid port value, use it.
@@ -590,17 +572,17 @@ send_spa_packet_http(const char *spa_data, const int sd_len,
             options->http_user_agent,
             options->http_proxy  /* hostname or IP */
         );
-        strlcpy(options->spa_server_str, options->http_proxy, MAX_SERVER_STR_LEN);
+        strlcpy(options->spa_server_str, options->http_proxy,
+                sizeof(options->spa_server_str));
     }
     free(spa_data_copy);
 
     if (options->test)
     {
-        if (options->verbose)
-           fprintf(stderr, "%s\n", http_buf);
+        log_msg(LOG_VERBOSITY_INFO, "%s", http_buf);
 
-        fprintf(stderr,
-            "Test mode enabled, SPA packet not actually sent.\n");
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "Test mode enabled, SPA packet not actually sent.");
         return 0;
     }
 
@@ -614,21 +596,24 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
 {
     int                 res, sd_len;
     char               *spa_data;
-
     struct sockaddr_in  saddr, daddr;
-
+    char                ip_str[INET_ADDRSTRLEN] = {0};  /* String used to contain the ip addres of an hostname */
+    struct addrinfo     hints;                          /* Structure used to set hints to reslove hostname */
 #ifdef WIN32
     WSADATA wsa_data;
 #endif
 
+    /* Initialize the hint buffer */
+    memset(&hints, 0 , sizeof(hints));
+
     /* Get our spa data here.
     */
     res = fko_get_spa_data(ctx, &spa_data);
 
     if(res != FKO_SUCCESS)
     {
-        fprintf(stderr,
-            "send_spa_packet: Error #%i from fko_get_spa_data: %s\n",
+        log_msg(LOG_VERBOSITY_ERROR,
+            "send_spa_packet: Error #%i from fko_get_spa_data: %s",
             res, fko_errstr(res)
         );
         return(-1);
@@ -642,15 +627,14 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
     res = WSAStartup( MAKEWORD(1,1), &wsa_data );
     if( res != 0 )
     {
-        fprintf(stderr, "Winsock initialization error %d\n", res );
+        log_msg(LOG_VERBOSITY_ERROR, "Winsock initialization error %d", res );
         return(-1);
     }
 #endif
 
     errno = 0;
 
-    if (options->verbose)
-        dump_transmit_options(options);
+    dump_transmit_options(options);
 
     if (options->spa_proto == FKO_PROTO_TCP || options->spa_proto == FKO_PROTO_UDP)
     {
@@ -684,20 +668,27 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
 
         if (saddr.sin_addr.s_addr == -1)
         {
-            fprintf(stderr, "Could not set source IP.\n");
-            exit(EXIT_FAILURE);
+            log_msg(LOG_VERBOSITY_ERROR, "Could not set source IP.");
+            return -1;
         }
 
-        /* Set destination address and port
+        /* Set destination port
         */
         daddr.sin_port = htons(options->spa_dst_port);
-        daddr.sin_addr.s_addr = inet_addr(options->spa_server_str);
 
-        if (daddr.sin_addr.s_addr == -1)
+        /* Set destination address. We use the default protocol to reslove
+         * the ip address */
+        hints.ai_family = AF_INET;
+
+        if (resolve_dest_adr(options->spa_server_str, &hints, ip_str, sizeof(ip_str)) != 0)
         {
-            fprintf(stderr, "Could not set destination IP.\n");
-            exit(EXIT_FAILURE);
+            log_msg(LOG_VERBOSITY_ERROR, "[*] Unable to resolve %s as an ip address",
+                    options->spa_server_str);
+            return -1;
         }
+        else;
+
+        daddr.sin_addr.s_addr = inet_addr(ip_str);
 
         if (options->spa_proto == FKO_PROTO_TCP_RAW)
         {
@@ -715,7 +706,7 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
     else
     {
         /* --DSS XXX: What to we really want to do here? */
-        fprintf(stderr, "%i is not a valid or supported protocol.\n",
+        log_msg(LOG_VERBOSITY_ERROR, "%i is not a valid or supported protocol.",
             options->spa_proto);
         res = -1;
     }
@@ -735,8 +726,8 @@ int write_spa_packet_data(fko_ctx_t ctx, const fko_cli_options_t *options)
 
     if(res != FKO_SUCCESS)
     {
-        fprintf(stderr,
-            "write_spa_packet_data: Error #%i from fko_get_spa_data: %s\n",
+        log_msg(LOG_VERBOSITY_ERROR,
+            "write_spa_packet_data: Error #%i from fko_get_spa_data: %s",
             res, fko_errstr(res)
         );
 
@@ -755,7 +746,7 @@ int write_spa_packet_data(fko_ctx_t ctx, const fko_cli_options_t *options)
 
     if(fp == NULL)
     {
-        perror("write_spa_packet_data: ");
+        log_msg(LOG_VERBOSITY_ERROR, "write_spa_packet_data: ", strerror(errno));
         return(-1);
     }
 
