@@ -88,32 +88,13 @@ hex_dump(const unsigned char *data, const int size)
     {
         for(i=0; i < 16-ln; i++)
             printf("   ");
+        if(ln < 8)
+            printf(" ");
 
         printf(" %s\n\n", ascii_str);
     }
 }
 
-/* Determine if a buffer contains only characters from the base64
- * encoding set
-*/
-int
-is_base64(const unsigned char *buf, const unsigned short int len)
-{
-    unsigned short int  i;
-    int                 rv = 1;
-
-    for(i=0; i<len; i++)
-    {
-        if(!(isalnum(buf[i]) || buf[i] == '/' || buf[i] == '+' || buf[i] == '='))
-        {
-            rv = 0;
-            break;
-        }
-    }
-
-    return rv;
-}
-
 int
 verify_file_perms_ownership(const char *file)
 {
