@@ -7,9 +7,9 @@
  *
  * Purpose: Common header file for fwknop client and server programs.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -83,6 +83,7 @@
   #define snprintf		_snprintf
   #define unlink		_unlink
   #define open			_open
+  #define fdopen        _fdopen
   #define close			_close
   #define write			_write
   #define O_WRONLY		_O_WRONLY
@@ -93,6 +94,16 @@
   #define S_IRUSR		_S_IREAD
   #define S_IWUSR		_S_IWRITE
   #define PATH_SEP      '\\'
+  // --DSS needed for VS versions before 2010
+  typedef __int8 int8_t;
+  typedef unsigned __int8 uint8_t;
+  typedef __int16 int16_t;
+  typedef unsigned __int16 uint16_t;
+  typedef __int32 int32_t;
+  typedef unsigned __int32 uint32_t;
+  typedef __int64 int64_t;
+  typedef unsigned __int64 uint64_t;
+
 #else
   #include <signal.h>
   #define PATH_SEP      '/'
@@ -100,6 +111,7 @@
 
 #include "fko.h"
 #include "fko_limits.h"
+#include "fko_util.h"
 
 /* Get our program version from VERSION (defined in config.h).
 */
@@ -133,6 +145,10 @@ enum {
 
 /* Some convenience macros */
 
+/* Get the number of elements of an array
+ */
+#define ARRAY_SIZE(t)   (sizeof(t) / sizeof(t[0]))
+
 /* Characters allowed between a config parameter and its value.
 */
 #define IS_CONFIG_PARAM_DELIMITER(x) (x == ' ' || x == '\t' || x == '=');
