@@ -7,9 +7,9 @@
  *
  * Purpose: Header for the fwknop cipher_funcs.c.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,7 +31,6 @@
 #ifndef CIPHER_FUNCS_H
 #define CIPHER_FUNCS_H 1
 
-#include "fko_common.h"
 #include "rijndael.h"
 #include "gpgme_funcs.h"
 
@@ -41,8 +40,15 @@
 */
 #define PREDICT_ENCSIZE(x) (1+(x>>4)+(x&0xf?1:0))<<4
 
-size_t rij_encrypt(unsigned char *in, size_t len, const char *key, unsigned char *out);
-size_t rij_decrypt(unsigned char *in, size_t len, const char *key, unsigned char *out);
+void get_random_data(unsigned char *data, const size_t len);
+size_t rij_encrypt(unsigned char *in, size_t len,
+    const char *key, const int key_len,
+    unsigned char *out, int encryption_mode);
+size_t rij_decrypt(unsigned char *in, size_t len,
+    const char *key, const int key_len,
+    unsigned char *out, int encryption_mode);
+int add_salted_str(fko_ctx_t ctx);
+int add_gpg_prefix(fko_ctx_t ctx);
 
 #endif /* CIPHER_FUNCS_H */
 
