@@ -1,17 +1,15 @@
-/*
- *****************************************************************************
+/**
+ * @file    digest.c
  *
- * File:    digest.c
+ * @author  Damien S. Stuart
  *
- * Author:  Damien S. Stuart
+ * @brief   Roll-up of the digests used by fwknop.
  *
- * Purpose: Roll-up of the digests used by fwknop.
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ *  License (GNU General Public License):
  *
- *  License (GNU Public License):
- *
- *  This library is free software; you can redistribute it and/or
+ *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
  *  of the License, or (at your option) any later version.
@@ -25,24 +23,40 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
- *
- *****************************************************************************
-*/
+ */
+
+#include "fko_common.h"
 #include "digest.h"
 #include "base64.h"
 
+
 /* Convert a raw digest into its hex string representation.
 */
 static void
-digest_to_hex(char *out, const unsigned char *in, const size_t size)
+digest_to_hex(char *out, size_t size_out, const unsigned char *in, const size_t size_in)
 {
     size_t i;
 
-    for(i=0; i<size; i++)
+    /* Assume the output buffer must be a NULL terminated string */
+    memset(out, 0, size_out);
+    size_out -= 1;
+
+    /* The hex string representation must be long enough */
+    if (size_out >= (size_in * 2))
     {
-        sprintf(out, "%02x", in[i]);
-        out += 2;
+        /* For each byte... */
+        for(i=0; i<size_in; i++)
+        {
+            /* Append the hex string to the output buffer */
+            snprintf(out, 2, "%02x", in[i]);
+
+            /* Moved the pointer on the output buffer to the next place  */
+            out += 2;
+        }
     }
+
+    /* Not enough space in the output buffer - Should not occur */
+    else;
 }
 
 /* Compute MD5 hash on in and store result in out.
@@ -60,12 +74,12 @@ md5(unsigned char *out, unsigned char *in, size_t size)
 /* Compute MD5 hash on in and store the hex string result in out.
 */
 void
-md5_hex(char *out, unsigned char *in, size_t size)
+md5_hex(char *out, size_t size_out, unsigned char *in, size_t size_in)
 {
-    uint8_t      md[MD5_DIGEST_LENGTH];
+    uint8_t      md[MD5_DIGEST_LEN];
 
-    md5(md, in, size);
-    digest_to_hex(out, md, MD5_DIGEST_LENGTH);
+    md5(md, in, size_in);
+    digest_to_hex(out, size_out, md, MD5_DIGEST_LEN);
 }
 
 /* Compute MD5 hash on in and store the base64 string result in out.
@@ -73,10 +87,10 @@ md5_hex(char *out, unsigned char *in, size_t size)
 void
 md5_base64(char *out, unsigned char *in, size_t size)
 {
-    uint8_t      md[MD5_DIGEST_LENGTH];
+    uint8_t      md[MD5_DIGEST_LEN];
 
     md5(md, in, size);
-    b64_encode(md, out, MD5_DIGEST_LENGTH);
+    b64_encode(md, out, MD5_DIGEST_LEN);
 
     strip_b64_eq(out);
 }
@@ -96,12 +110,12 @@ sha1(unsigned char *out, unsigned char *in, size_t size)
 /* Compute SHA1 hash on in and store the hex string result in out.
 */
 void
-sha1_hex(char *out, unsigned char *in, size_t size)
+sha1_hex(char *out, size_t size_out, unsigned char *in, size_t size_in)
 {
-    uint8_t       md[SHA1_DIGEST_LENGTH];
+    uint8_t       md[SHA1_DIGEST_LEN];
 
-    sha1(md, in, size);
-    digest_to_hex(out, md, SHA1_DIGEST_LENGTH);
+    sha1(md, in, size_in);
+    digest_to_hex(out, size_out, md, SHA1_DIGEST_LEN);
 }
 
 /* Compute SHA1 hash on in and store the base64 string result in out.
@@ -109,10 +123,10 @@ sha1_hex(char *out, unsigned char *in, size_t size)
 void
 sha1_base64(char *out, unsigned char *in, size_t size)
 {
-    uint8_t       md[SHA1_DIGEST_LENGTH];
+    uint8_t       md[SHA1_DIGEST_LEN];
 
     sha1(md, in, size);
-    b64_encode(md, out, SHA1_DIGEST_LENGTH);
+    b64_encode(md, out, SHA1_DIGEST_LEN);
 
     strip_b64_eq(out);
 }
@@ -132,12 +146,12 @@ sha256(unsigned char *out, unsigned char *in, size_t size)
 /* Compute SHA256 hash on in and store the hex string result in out.
 */
 void
-sha256_hex(char *out, unsigned char *in, size_t size)
+sha256_hex(char *out, size_t size_out, unsigned char *in, size_t size_in)
 {
-    uint8_t       md[SHA256_DIGEST_LENGTH];
+    uint8_t       md[SHA256_DIGEST_LEN];
 
-    sha256(md, in, size);
-    digest_to_hex(out, md, SHA256_DIGEST_LENGTH);
+    sha256(md, in, size_in);
+    digest_to_hex(out, size_out, md, SHA256_DIGEST_LEN);
 }
 
 /* Compute SHA256 hash on in and store the base64 string result in out.
@@ -145,10 +159,10 @@ sha256_hex(char *out, unsigned char *in, size_t size)
 void
 sha256_base64(char *out, unsigned char *in, size_t size)
 {
-    uint8_t       md[SHA256_DIGEST_LENGTH];
+    uint8_t       md[SHA256_DIGEST_LEN];
 
     sha256(md, in, size);
-    b64_encode(md, out, SHA256_DIGEST_LENGTH);
+    b64_encode(md, out, SHA256_DIGEST_LEN);
 
     strip_b64_eq(out);
 }
@@ -168,12 +182,12 @@ sha384(unsigned char *out, unsigned char *in, size_t size)
 /* Compute SHA384 hash on in and store the hex string result in out.
 */
 void
-sha384_hex(char *out, unsigned char *in, size_t size)
+sha384_hex(char *out, size_t size_out, unsigned char *in, size_t size_in)
 {
-    uint8_t       md[SHA384_DIGEST_LENGTH];
+    uint8_t       md[SHA384_DIGEST_LEN];
 
-    sha384(md, in, size);
-    digest_to_hex(out, md, SHA384_DIGEST_LENGTH);
+    sha384(md, in, size_in);
+    digest_to_hex(out, size_out, md, SHA384_DIGEST_LEN);
 }
 
 /* Compute SHA384 hash on in and store the base64 string result in out.
@@ -181,10 +195,10 @@ sha384_hex(char *out, unsigned char *in, size_t size)
 void
 sha384_base64(char *out, unsigned char *in, size_t size)
 {
-    uint8_t       md[SHA384_DIGEST_LENGTH];
+    uint8_t       md[SHA384_DIGEST_LEN];
 
     sha384(md, in, size);
-    b64_encode(md, out, SHA384_DIGEST_LENGTH);
+    b64_encode(md, out, SHA384_DIGEST_LEN);
 
     strip_b64_eq(out);
 }
@@ -204,12 +218,12 @@ sha512(unsigned char *out, unsigned char *in, size_t size)
 /* Compute SHA512 hash on in and store the hex string result in out.
 */
 void
-sha512_hex(char *out, unsigned char *in, size_t size)
+sha512_hex(char *out, size_t size_out, unsigned char *in, size_t size_in)
 {
-    uint8_t       md[SHA512_DIGEST_LENGTH];
+    uint8_t       md[SHA512_DIGEST_LEN];
 
-    sha512(md, in, size);
-    digest_to_hex(out, md, SHA512_DIGEST_LENGTH);
+    sha512(md, in, size_in);
+    digest_to_hex(out, size_out, md, SHA512_DIGEST_LEN);
 }
 
 /* Compute SHA512 hash on in and store the base64 string result in out.
@@ -217,10 +231,10 @@ sha512_hex(char *out, unsigned char *in, size_t size)
 void
 sha512_base64(char *out, unsigned char *in, size_t size)
 {
-    uint8_t       md[SHA512_DIGEST_LENGTH];
+    uint8_t       md[SHA512_DIGEST_LEN];
 
     sha512(md, in, size);
-    b64_encode(md, out, SHA512_DIGEST_LENGTH);
+    b64_encode(md, out, SHA512_DIGEST_LEN);
 
     strip_b64_eq(out);
 }
