@@ -7,9 +7,9 @@
  *
  * Purpose: Header for the fwknop digest.c.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,8 +31,6 @@
 #ifndef DIGEST_H
 #define DIGEST_H 1
 
-#include "fko_common.h"
-
 #include "md5.h"
 #include "sha1.h"
 #include "sha2.h"
@@ -41,28 +39,20 @@
 */
 #define MD_HEX_SIZE(x) x * 2
 
-/* Predefined base64 encoded digest sizes.
-*/
-#define MD5_B64_LENGTH      22
-#define SHA1_B64_LENGTH     27
-#define SHA256_B64_LENGTH   43
-#define SHA384_B64_LENGTH   64
-#define SHA512_B64_LENGTH   86
-
 void md5(unsigned char* out, unsigned char* in, size_t size);
-void md5_hex(char* out, unsigned char* in, size_t size);
+void md5_hex(char* out, size_t size_out, unsigned char* in, size_t size);
 void md5_base64(char* out, unsigned char* in, size_t size);
 void sha1(unsigned char* out, unsigned char* in, size_t size);
-void sha1_hex(char* out, unsigned char* in, size_t size);
+void sha1_hex(char* out, size_t size_out, unsigned char* in, size_t size);
 void sha1_base64(char* out, unsigned char* in, size_t size);
 void sha256(unsigned char* out, unsigned char* in, size_t size);
-void sha256_hex(char* out, unsigned char* in, size_t size);
+void sha256_hex(char* out, size_t size_out, unsigned char* in, size_t size);
 void sha256_base64(char* out, unsigned char* in, size_t size);
 void sha384(unsigned char* out, unsigned char* in, size_t size);
-void sha384_hex(char* out, unsigned char* in, size_t size);
+void sha384_hex(char* out, size_t size_out, unsigned char* in, size_t size);
 void sha384_base64(char* out, unsigned char* in, size_t size);
 void sha512(unsigned char* out, unsigned char* in, size_t size);
-void sha512_hex(char* out, unsigned char* in, size_t size);
+void sha512_hex(char* out, size_t size_out, unsigned char* in, size_t size);
 void sha512_base64(char* out, unsigned char* in, size_t size);
 
 #endif /* DIGEST_H */
