@@ -7,9 +7,9 @@
  *
  * Purpose: fko context definition.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -33,6 +33,10 @@
 
 #include "fko_common.h"
 
+#if HAVE_LIBGPGME
+  #include <gpgme.h>
+#endif
+ 
 #if HAVE_LIBGPGME
 /* Stucture to hold a list of the gpg signature information
  * we are interested in.
@@ -51,6 +55,7 @@ typedef struct fko_gpg_sig *fko_gpg_sig_t;
 /* The pieces we need to make an FKO  SPA data packet.
 */
 struct fko_context {
+
     /* FKO SPA user-definable message data */
     char           *rand_val;
     char           *username;
@@ -64,23 +69,33 @@ struct fko_context {
     /* FKO SPA user-settable message encoding types */
     short  digest_type;
     short  encryption_type;
+    int    encryption_mode;
+    short  hmac_type;
 
     /* Computed or predefined data */
     char           *version;
     char           *digest;
+    int             digest_len;
 
     /* Digest of raw encrypted/base64 data - this is used
      * for replay attack detection
     */
     char           *raw_digest;
     short           raw_digest_type;
+    int             raw_digest_len;
 
     /* Computed processed data (encodings, etc.) */
     char           *encoded_msg;
+    int             encoded_msg_len;
     char           *encrypted_msg;
+    int             encrypted_msg_len;
+    char           *msg_hmac;
+    int             msg_hmac_len;
+    int             added_salted_str;
+    int             added_gpg_prefix;
 
     /* State info */
-    unsigned short  state;
+    unsigned int    state;
     unsigned char   initval;
 
 #if HAVE_LIBGPGME
