@@ -7,9 +7,9 @@
  *
  * Purpose: Decode an FKO SPA message after decryption.
  *
- * Copyright 2009-2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -40,23 +40,16 @@ int
 fko_decode_spa_data(fko_ctx_t ctx)
 {
     char       *tbuf, *ndx, *tmp;
-    int         t_size, i;
+    int         t_size, i, is_err;
 
-    /* Check for required data.
-    */
-    if(ctx->encoded_msg == NULL || strnlen(ctx->encoded_msg,
-            MAX_SPA_ENCODED_MSG_SIZE) < MIN_SPA_ENCODED_MSG_SIZE)
-        return(FKO_ERROR_INVALID_DATA);
-
-    if(strnlen(ctx->encoded_msg,
-            MAX_SPA_ENCODED_MSG_SIZE) == MAX_SPA_ENCODED_MSG_SIZE)
-        return(FKO_ERROR_INVALID_DATA);
+    if (! is_valid_encoded_msg_len(ctx->encoded_msg_len))
+        return(FKO_ERROR_INVALID_DATA_DECODE_MSGLEN_VALIDFAIL);
 
     /* Make sure there are no non-ascii printable chars
     */
     for (i=0; i < (int)strnlen(ctx->encoded_msg, MAX_SPA_ENCODED_MSG_SIZE); i++)
         if(isprint(ctx->encoded_msg[i]) == 0)
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_NON_ASCII);
 
     /* Make sure there are enough fields in the SPA packet
      * delimited with ':' chars
@@ -72,36 +65,47 @@ fko_decode_spa_data(fko_ctx_t ctx)
     }
 
     if (i < MIN_SPA_FIELDS)
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_LT_MIN_FIELDS);
 
-    t_size = strlen(ndx);
+    t_size = strnlen(ndx, SHA512_B64_LEN+1);
 
     switch(t_size)
     {
-        case MD5_B64_LENGTH:
+        case MD5_B64_LEN:
             ctx->digest_type = FKO_DIGEST_MD5;
+            ctx->digest_len  = MD5_B64_LEN;
             break;
 
-        case SHA1_B64_LENGTH:
+        case SHA1_B64_LEN:
             ctx->digest_type = FKO_DIGEST_SHA1;
+            ctx->digest_len  = SHA1_B64_LEN;
             break;
 
-        case SHA256_B64_LENGTH:
+        case SHA256_B64_LEN:
             ctx->digest_type = FKO_DIGEST_SHA256;
+            ctx->digest_len  = SHA256_B64_LEN;
             break;
 
-        case SHA384_B64_LENGTH:
+        case SHA384_B64_LEN:
             ctx->digest_type = FKO_DIGEST_SHA384;
+            ctx->digest_len  = SHA384_B64_LEN;
             break;
 
-        case SHA512_B64_LENGTH:
+        case SHA512_B64_LEN:
             ctx->digest_type = FKO_DIGEST_SHA512;
+            ctx->digest_len  = SHA512_B64_LEN;
             break;
 
         default: /* Invalid or unsupported digest */
             return(FKO_ERROR_INVALID_DIGEST_TYPE);
     }
 
+    if (ctx->encoded_msg_len - t_size < 0)
+        return(FKO_ERROR_INVALID_DATA_DECODE_ENC_MSG_LEN_MT_T_SIZE);
+
+    if(ctx->digest != NULL)
+        free(ctx->digest);
+
     /* Copy the digest into the context and terminate the encoded data
      * at that point so the original digest is not part of the
      * encoded string.
@@ -114,6 +118,8 @@ fko_decode_spa_data(fko_ctx_t ctx)
     */
     bzero((ndx-1), t_size);
 
+    ctx->encoded_msg_len -= t_size+1;
+
     /* Make a tmp bucket for processing base64 encoded data and
      * other general use.
     */
@@ -126,31 +132,30 @@ fko_decode_spa_data(fko_ctx_t ctx)
     switch(ctx->digest_type)
     {
         case FKO_DIGEST_MD5:
-            md5_base64(tbuf, (unsigned char*)ctx->encoded_msg, strlen(ctx->encoded_msg));
+            md5_base64(tbuf, (unsigned char*)ctx->encoded_msg, ctx->encoded_msg_len);
             break;
 
         case FKO_DIGEST_SHA1:
-            sha1_base64(tbuf, (unsigned char*)ctx->encoded_msg, strlen(ctx->encoded_msg));
+            sha1_base64(tbuf, (unsigned char*)ctx->encoded_msg, ctx->encoded_msg_len);
             break;
 
         case FKO_DIGEST_SHA256:
-            sha256_base64(tbuf, (unsigned char*)ctx->encoded_msg, strlen(ctx->encoded_msg));
+            sha256_base64(tbuf, (unsigned char*)ctx->encoded_msg, ctx->encoded_msg_len);
             break;
 
         case FKO_DIGEST_SHA384:
-            sha384_base64(tbuf, (unsigned char*)ctx->encoded_msg, strlen(ctx->encoded_msg));
+            sha384_base64(tbuf, (unsigned char*)ctx->encoded_msg, ctx->encoded_msg_len);
             break;
 
         case FKO_DIGEST_SHA512:
-            sha512_base64(tbuf, (unsigned char*)ctx->encoded_msg, strlen(ctx->encoded_msg));
+            sha512_base64(tbuf, (unsigned char*)ctx->encoded_msg, ctx->encoded_msg_len);
             break;
-
     }
 
     /* We give up here if the computed digest does not match the
      * digest in the message data.
     */
-    if(strncmp(ctx->digest, tbuf, t_size))
+    if(constant_runtime_cmp(ctx->digest, tbuf, t_size) != 0)
     {
         free(tbuf);
         return(FKO_ERROR_DIGEST_VERIFICATION_FAILED);
@@ -165,9 +170,12 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < FKO_RAND_VAL_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_RAND_MISSING);
     }
 
+    if(ctx->rand_val != NULL)
+        free(ctx->rand_val);
+
     ctx->rand_val = calloc(1, FKO_RAND_VAL_SIZE+1);
     if(ctx->rand_val == NULL)
     {
@@ -183,17 +191,20 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < 1)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_USERNAME_MISSING);
     }
 
     if (t_size > MAX_SPA_USERNAME_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_USERNAME_TOOBIG);
     }
 
     strlcpy(tbuf, ndx, t_size+1);
 
+    if(ctx->username != NULL)
+        free(ctx->username);
+
     ctx->username = malloc(t_size+1); /* Yes, more than we need */
     if(ctx->username == NULL)
     {
@@ -204,12 +215,12 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if(b64_decode(tbuf, (unsigned char*)ctx->username) < 0)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_USERNAME_DECODEFAIL);
     }
     if(validate_username(ctx->username) != FKO_SUCCESS)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_USERNAME_VALIDFAIL);
     }
 
     /* Extract the timestamp value.
@@ -218,18 +229,24 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < 1)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_MISSING);
     }
 
     if (t_size > MAX_SPA_TIMESTAMP_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_TOOBIG);
     }
 
     strlcpy(tbuf, ndx, t_size+1);
 
-    ctx->timestamp = (unsigned int)atoi(tbuf);
+    ctx->timestamp = (unsigned int) strtol_wrapper(tbuf,
+            0, -1, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        free(tbuf);
+        return(FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_DECODEFAIL);
+    }
 
     /* Extract the version string.
     */
@@ -237,15 +254,18 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < 1)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_VERSION_MISSING);
     }
 
     if (t_size > MAX_SPA_VERSION_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_VERSION_TOOBIG);
     }
 
+    if(ctx->version != NULL)
+        free(ctx->version);
+
     ctx->version = malloc(t_size+1);
     if(ctx->version == NULL)
     {
@@ -261,23 +281,23 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < 1)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_MISSING);
     }
 
     if (t_size > MAX_SPA_MESSAGE_TYPE_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_TOOBIG);
     }
 
     strlcpy(tbuf, ndx, t_size+1);
 
-    ctx->message_type = (unsigned int)atoi(tbuf);
-
-    if(ctx->message_type < 0 || ctx->message_type >= FKO_LAST_MSG_TYPE)
+    ctx->message_type = strtol_wrapper(tbuf, 0,
+            FKO_LAST_MSG_TYPE, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_DECODEFAIL);
     }
 
     /* Extract the SPA message string.
@@ -286,17 +306,20 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if((t_size = strcspn(ndx, ":")) < 1)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_MISSING);
     }
 
     if (t_size > MAX_SPA_MESSAGE_SIZE)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_TOOBIG);
     }
 
     strlcpy(tbuf, ndx, t_size+1);
 
+    if(ctx->message != NULL)
+        free(ctx->message);
+
     ctx->message = malloc(t_size+1); /* Yes, more than we need */
     if(ctx->message == NULL)
     {
@@ -307,7 +330,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
     if(b64_decode(tbuf, (unsigned char*)ctx->message) < 0)
     {
         free(tbuf);
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_DECODEFAIL);
     }
 
     if(ctx->message_type == FKO_COMMAND_MSG)
@@ -317,7 +340,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
         if(validate_cmd_msg(ctx->message) != FKO_SUCCESS)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_VALIDFAIL);
         }
     }
     else
@@ -327,7 +350,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
         if(validate_access_msg(ctx->message) != FKO_SUCCESS)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_ACCESS_VALIDFAIL);
         }
     }
 
@@ -342,17 +365,20 @@ fko_decode_spa_data(fko_ctx_t ctx)
         if((t_size = strcspn(ndx, ":")) < 1)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_MISSING);
         }
 
         if (t_size > MAX_SPA_MESSAGE_SIZE)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_TOOBIG);
         }
 
         strlcpy(tbuf, ndx, t_size+1);
 
+        if(ctx->nat_access != NULL)
+            free(ctx->nat_access);
+
         ctx->nat_access = malloc(t_size+1); /* Yes, more than we need */
         if(ctx->nat_access == NULL)
         {
@@ -363,13 +389,13 @@ fko_decode_spa_data(fko_ctx_t ctx)
         if(b64_decode(tbuf, (unsigned char*)ctx->nat_access) < 0)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_DECODEFAIL);
         }
 
         if(validate_nat_access_msg(ctx->nat_access) != FKO_SUCCESS)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_VALIDFAIL);
         }
     }
 
@@ -381,7 +407,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
         if (t_size > MAX_SPA_MESSAGE_SIZE)
         {
             free(tbuf);
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_DECODE_SRVAUTH_MISSING);
         }
 
         /* There is data, but what is it?
@@ -394,6 +420,9 @@ fko_decode_spa_data(fko_ctx_t ctx)
         {
             strlcpy(tbuf, ndx, t_size+1);
 
+            if(ctx->server_auth != NULL)
+                free(ctx->server_auth);
+
             ctx->server_auth = malloc(t_size+1); /* Yes, more than we need */
             if(ctx->server_auth == NULL)
             {
@@ -404,7 +433,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
             if(b64_decode(tbuf, (unsigned char*)ctx->server_auth) < 0)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_SRVAUTH_DECODEFAIL);
             }
 
             /* At this point we should be done.
@@ -431,13 +460,16 @@ fko_decode_spa_data(fko_ctx_t ctx)
             if (t_size > MAX_SPA_MESSAGE_SIZE)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_EXTRA_TOOBIG);
             }
 
             /* Looks like we have both, so assume this is the 
             */
             strlcpy(tbuf, ndx, t_size+1);
 
+            if(ctx->server_auth != NULL)
+                free(ctx->server_auth);
+
             ctx->server_auth = malloc(t_size+1); /* Yes, more than we need */
             if(ctx->server_auth == NULL)
             {
@@ -448,7 +480,7 @@ fko_decode_spa_data(fko_ctx_t ctx)
             if(b64_decode(tbuf, (unsigned char*)ctx->server_auth) < 0)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_EXTRA_DECODEFAIL);
             }
 
             ndx += t_size + 1;
@@ -463,12 +495,12 @@ fko_decode_spa_data(fko_ctx_t ctx)
             if((t_size = strlen(ndx)) < 1)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_MISSING);
             }
             if (t_size > MAX_SPA_MESSAGE_SIZE)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_TOOBIG);
             }
 
             /* Should be a number only.
@@ -476,10 +508,16 @@ fko_decode_spa_data(fko_ctx_t ctx)
             if(strspn(ndx, "0123456789") != t_size)
             {
                 free(tbuf);
-                return(FKO_ERROR_INVALID_DATA);
+                return(FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_VALIDFAIL);
             }
 
-            ctx->client_timeout = (unsigned int)atoi(ndx);
+            ctx->client_timeout = (unsigned int) strtol_wrapper(ndx, 0,
+                    (2 << 15), NO_EXIT_UPON_ERR, &is_err);
+            if(is_err != FKO_SUCCESS)
+            {
+                free(tbuf);
+                return(FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_DECODEFAIL);
+            }
         }
     }
 
