@@ -45,7 +45,7 @@ append_b64(char* tbuf, char *str)
     char *bs;
 
     if(len >= MAX_SPA_ENCODED_MSG_SIZE)
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_ENCODE_MESSAGE_TOOBIG);
 
     bs = malloc(((len/3)*4)+8);
     if(bs == NULL)
@@ -201,7 +201,7 @@ fko_encode_spa_data(fko_ctx_t ctx)
     ctx->encoded_msg_len = strnlen(ctx->encoded_msg, MAX_SPA_ENCODED_MSG_SIZE);
 
     if(! is_valid_encoded_msg_len(ctx->encoded_msg_len))
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_ENCODE_MSGLEN_VALIDFAIL);
 
     /* At this point we can compute the digest for this SPA data.
     */
@@ -230,6 +230,9 @@ fko_get_encoded_data(fko_ctx_t ctx, char **enc_msg)
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+    if(enc_msg == NULL)
+        return(FKO_ERROR_INVALID_DATA);
+
     *enc_msg = ctx->encoded_msg;
 
     return(FKO_SUCCESS);
