@@ -70,7 +70,7 @@ fko_set_rand_value(fko_ctx_t ctx, const char * const new_val)
     if(new_val != NULL)
     {
         if(strnlen(new_val, FKO_RAND_VAL_SIZE+1) != FKO_RAND_VAL_SIZE)
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_RAND_LEN_VALIDFAIL);
 
         if(ctx->rand_val != NULL)
             free(ctx->rand_val);
@@ -152,6 +152,9 @@ fko_get_rand_value(fko_ctx_t ctx, char **rand_value)
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+    if(rand_value == NULL)
+        return(FKO_ERROR_INVALID_DATA);
+
     *rand_value = ctx->rand_val;
 
     return(FKO_SUCCESS);
