@@ -128,6 +128,9 @@ fko_get_username(fko_ctx_t ctx, char **username)
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+    if(username == NULL)
+        return(FKO_ERROR_INVALID_DATA);
+
     *username = ctx->username;
 
     return(FKO_SUCCESS);
@@ -139,17 +142,17 @@ validate_username(const char *username)
     int i;
 
     if(username == NULL || strnlen(username, MAX_SPA_USERNAME_SIZE) == 0)
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_USER_MISSING);
 
     /* Make sure it is just alpha-numeric chars, dashes, dots, and underscores
     */
     if(isalnum(username[0]) == 0)
-        return(FKO_ERROR_INVALID_DATA);
+        return(FKO_ERROR_INVALID_DATA_USER_FIRSTCHAR_VALIDFAIL);
 
     for (i=1; i < (int)strnlen(username, MAX_SPA_USERNAME_SIZE); i++)
         if((isalnum(username[i]) == 0)
                 && username[i] != '-' && username[i] != '_' && username[i] != '.')
-            return(FKO_ERROR_INVALID_DATA);
+            return(FKO_ERROR_INVALID_DATA_USER_REMCHAR_VALIDFAIL);
 
     return FKO_SUCCESS;
 }
