@@ -13,7 +13,7 @@
  * Copyright (c) Cryptoapi developers.
  * Copyright (c) 2002 James Morris <jmorris@intercode.com.au>
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -35,9 +35,12 @@
 #ifndef MD5_H
 #define MD5_H 1
 
-#include "fko_common.h"
+#include "common.h"
 
-#define MD5_DIGEST_LENGTH 16
+#define MD5_BLOCK_LEN       64
+#define MD5_DIGEST_LEN      16
+#define MD5_DIGEST_STR_LEN  (MD5_DIGEST_LEN * 2 + 1)
+#define MD5_B64_LEN         22
 
 typedef struct _MD5Context {
         uint32_t buf[4];
