@@ -9,7 +9,7 @@
  *
  * Currently maintained by brian d foy, <bdfoy@cpan.org>
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -33,17 +33,19 @@
  * 128, 192, or 256 bits, designed by Joan Daemen and Vincent Rijmen.  See
  * http://www.esat.kuleuven.ac.be/~rijmen/rijndael/ for details.
  */
-
 #ifndef RIJNDAEL_H
 #define RIJNDAEL_H 1
 
-#include "fko_common.h"
+#include "common.h"
 
 /* Other block sizes and key lengths are possible, but in the context of
- * the ssh protocols, 256 bits is the default. 
+ * the ssh protocols, 256 bits is the default.
  */
 #define RIJNDAEL_BLOCKSIZE 16
 #define RIJNDAEL_KEYSIZE   32
+#define RIJNDAEL_MIN_KEYSIZE 16
+#define RIJNDAEL_MAX_KEYSIZE 32
+#define SALT_LEN 8
 
 #define     MODE_ECB        1    /*  Are we ciphering in ECB mode?   */
 #define     MODE_CBC        2    /*  Are we ciphering in CBC mode?   */
@@ -54,18 +56,15 @@
 
 /* Allow keys of size 128 <= bits <= 256 */
 
-#define RIJNDAEL_MIN_KEYSIZE 16
-#define RIJNDAEL_MAX_KEYSIZE 32
-
 typedef struct {
   uint32_t keys[60];		/* maximum size of key schedule */
   uint32_t ikeys[60];		/* inverse key schedule */
   int nrounds;			/* number of rounds to use for our key size */
   int mode;			    /* encryption mode */
   /* Added by DSS */
-  uint8_t key[32];
-  uint8_t iv[16];
-  uint8_t salt[8];
+  uint8_t key[RIJNDAEL_MAX_KEYSIZE];
+  uint8_t iv[RIJNDAEL_BLOCKSIZE];
+  uint8_t salt[SALT_LEN];
 } RIJNDAEL_context;
 
 /* This basically performs Rijndael's key scheduling algorithm, as it's the
@@ -74,10 +73,11 @@ typedef struct {
  * bits).  If a value other than these three is specified, the key will be
  * truncated to the closest value less than the key size specified, e.g.
  * specifying 7 will use only the first 6 bytes of the key given.  DO NOT
- * PASS A VALUE LESS THAN 16 TO KEYSIZE! 
+ * PASS A VALUE LESS THAN 16 TO KEYSIZE!
  */
 void
-rijndael_setup(RIJNDAEL_context *ctx, size_t keysize, const uint8_t *key);
+rijndael_setup(RIJNDAEL_context *ctx,
+    const size_t keysize, const uint8_t *key);
 
 /*
  * rijndael_encrypt()
@@ -103,7 +103,7 @@ rijndael_encrypt(RIJNDAEL_context *context,
  *
  * Before this function can be used, rijndael_setup() must be used in order
  * to set up the key schedule required for the decryption algorithm.
- * 
+ *
  * This function always decrypts 16 bytes of ciphertext to 16 bytes of
  * plaintext.  The memory areas of the plaintext and the ciphertext can
  * overlap.
