@@ -10,7 +10,7 @@
  *
  * Copyright (C) 2001 Rafael R. Sevilla <sevillar@team.ph.inter.net>
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@
 #ifndef SHA1_H
 #define SHA1_H 1
 
-#include "fko_common.h"
+#include "common.h"
 
 #ifdef WIN32
   #define BYTEORDER 1234
@@ -45,7 +45,10 @@
 #endif
 
 #define SHA1_BLOCKSIZE      64
-#define SHA1_DIGEST_LENGTH  20
+#define SHA1_BLOCK_LEN      SHA1_BLOCKSIZE
+#define SHA1_DIGEST_LEN     20
+#define SHA1_DIGEST_STR_LEN (SHA1_DIGEST_LEN * 2 + 1)
+#define SHA1_B64_LEN        27
 
 typedef struct {
     uint32_t    digest[8];
@@ -58,6 +61,6 @@ typedef struct {
 */
 void sha1_init(SHA1_INFO *sha1_info);
 void sha1_update(SHA1_INFO *sha1_info, uint8_t *buffer, int count);
-void sha1_final(uint8_t digest[SHA1_DIGEST_LENGTH], SHA1_INFO *sha1_info);
+void sha1_final(uint8_t digest[SHA1_DIGEST_LEN], SHA1_INFO *sha1_info);
 
 #endif /* SHA1_H */
