@@ -43,7 +43,7 @@
 extern "C" {
 #endif
 
-#include "fko_common.h"
+#include "common.h"
 
 /*
  * Import u_intXX_t size_t type definitions from system headers.  You
@@ -67,15 +67,20 @@ extern "C" {
 #endif
 
 /*** SHA-256/384/512 Various Length Definitions ***********************/
-#define SHA256_BLOCK_LENGTH		64
-#define SHA256_DIGEST_LENGTH		32
-#define SHA256_DIGEST_STRING_LENGTH	(SHA256_DIGEST_LENGTH * 2 + 1)
-#define SHA384_BLOCK_LENGTH		128
-#define SHA384_DIGEST_LENGTH		48
-#define SHA384_DIGEST_STRING_LENGTH	(SHA384_DIGEST_LENGTH * 2 + 1)
-#define SHA512_BLOCK_LENGTH		128
-#define SHA512_DIGEST_LENGTH		64
-#define SHA512_DIGEST_STRING_LENGTH	(SHA512_DIGEST_LENGTH * 2 + 1)
+#define SHA256_BLOCK_LEN        64
+#define SHA256_DIGEST_LEN       32
+#define SHA256_DIGEST_STR_LEN   (SHA256_DIGEST_LEN * 2 + 1)
+#define SHA256_B64_LEN          43
+
+#define SHA384_BLOCK_LEN        128
+#define SHA384_DIGEST_LEN       48
+#define SHA384_DIGEST_STR_LEN   (SHA384_DIGEST_LEN * 2 + 1)
+#define SHA384_B64_LEN          64
+
+#define SHA512_BLOCK_LEN        128
+#define SHA512_DIGEST_LEN       64
+#define SHA512_DIGEST_STR_LEN   (SHA512_DIGEST_LEN * 2 + 1)
+#define SHA512_B64_LEN          86
 
 
 /*** SHA-256/384/512 Context Structures *******************************/
@@ -109,12 +114,12 @@ typedef unsigned long long u_int64_t;	/* 8-bytes (64-bits) */
 typedef struct _SHA256_CTX {
 	uint32_t	state[8];
 	uint64_t	bitcount;
-	uint8_t	buffer[SHA256_BLOCK_LENGTH];
+	uint8_t	buffer[SHA256_BLOCK_LEN];
 } SHA256_CTX;
 typedef struct _SHA512_CTX {
 	uint64_t	state[8];
 	uint64_t	bitcount[2];
-	uint8_t	buffer[SHA512_BLOCK_LENGTH];
+	uint8_t	buffer[SHA512_BLOCK_LEN];
 } SHA512_CTX;
 
 #else /* SHA2_USE_INTTYPES_H */
@@ -122,12 +127,12 @@ typedef struct _SHA512_CTX {
 typedef struct _SHA256_CTX {
 	u_int32_t	state[8];
 	u_int64_t	bitcount;
-	u_int8_t	buffer[SHA256_BLOCK_LENGTH];
+	u_int8_t	buffer[SHA256_BLOCK_LEN];
 } SHA256_CTX;
 typedef struct _SHA512_CTX {
 	u_int64_t	state[8];
 	u_int64_t	bitcount[2];
-	u_int8_t	buffer[SHA512_BLOCK_LENGTH];
+	u_int8_t	buffer[SHA512_BLOCK_LEN];
 } SHA512_CTX;
 
 #endif /* SHA2_USE_INTTYPES_H */
@@ -141,41 +146,41 @@ typedef SHA512_CTX SHA384_CTX;
 
 void SHA256_Init(SHA256_CTX *);
 void SHA256_Update(SHA256_CTX*, const uint8_t*, size_t);
-void SHA256_Final(uint8_t[SHA256_DIGEST_LENGTH], SHA256_CTX*);
-char* SHA256_End(SHA256_CTX*, char[SHA256_DIGEST_STRING_LENGTH]);
-char* SHA256_Data(const uint8_t*, size_t, char[SHA256_DIGEST_STRING_LENGTH]);
+void SHA256_Final(uint8_t[SHA256_DIGEST_LEN], SHA256_CTX*);
+char* SHA256_End(SHA256_CTX*, char[SHA256_DIGEST_STR_LEN]);
+char* SHA256_Data(const uint8_t*, size_t, char[SHA256_DIGEST_STR_LEN]);
 
 void SHA384_Init(SHA384_CTX*);
 void SHA384_Update(SHA384_CTX*, const uint8_t*, size_t);
-void SHA384_Final(uint8_t[SHA384_DIGEST_LENGTH], SHA384_CTX*);
-char* SHA384_End(SHA384_CTX*, char[SHA384_DIGEST_STRING_LENGTH]);
-char* SHA384_Data(const uint8_t*, size_t, char[SHA384_DIGEST_STRING_LENGTH]);
+void SHA384_Final(uint8_t[SHA384_DIGEST_LEN], SHA384_CTX*);
+char* SHA384_End(SHA384_CTX*, char[SHA384_DIGEST_STR_LEN]);
+char* SHA384_Data(const uint8_t*, size_t, char[SHA384_DIGEST_STR_LEN]);
 
 void SHA512_Init(SHA512_CTX*);
 void SHA512_Update(SHA512_CTX*, const uint8_t*, size_t);
-void SHA512_Final(uint8_t[SHA512_DIGEST_LENGTH], SHA512_CTX*);
-char* SHA512_End(SHA512_CTX*, char[SHA512_DIGEST_STRING_LENGTH]);
-char* SHA512_Data(const uint8_t*, size_t, char[SHA512_DIGEST_STRING_LENGTH]);
+void SHA512_Final(uint8_t[SHA512_DIGEST_LEN], SHA512_CTX*);
+char* SHA512_End(SHA512_CTX*, char[SHA512_DIGEST_STR_LEN]);
+char* SHA512_Data(const uint8_t*, size_t, char[SHA512_DIGEST_STR_LEN]);
 
 #else /* SHA2_USE_INTTYPES_H */
 
 void SHA256_Init(SHA256_CTX *);
 void SHA256_Update(SHA256_CTX*, const u_int8_t*, size_t);
-void SHA256_Final(u_int8_t[SHA256_DIGEST_LENGTH], SHA256_CTX*);
-char* SHA256_End(SHA256_CTX*, char[SHA256_DIGEST_STRING_LENGTH]);
-char* SHA256_Data(const u_int8_t*, size_t, char[SHA256_DIGEST_STRING_LENGTH]);
+void SHA256_Final(u_int8_t[SHA256_DIGEST_LEN], SHA256_CTX*);
+char* SHA256_End(SHA256_CTX*, char[SHA256_DIGEST_STR_LEN]);
+char* SHA256_Data(const u_int8_t*, size_t, char[SHA256_DIGEST_STR_LEN]);
 
 void SHA384_Init(SHA384_CTX*);
 void SHA384_Update(SHA384_CTX*, const u_int8_t*, size_t);
-void SHA384_Final(u_int8_t[SHA384_DIGEST_LENGTH], SHA384_CTX*);
-char* SHA384_End(SHA384_CTX*, char[SHA384_DIGEST_STRING_LENGTH]);
-char* SHA384_Data(const u_int8_t*, size_t, char[SHA384_DIGEST_STRING_LENGTH]);
+void SHA384_Final(u_int8_t[SHA384_DIGEST_LEN], SHA384_CTX*);
+char* SHA384_End(SHA384_CTX*, char[SHA384_DIGEST_STR_LEN]);
+char* SHA384_Data(const u_int8_t*, size_t, char[SHA384_DIGEST_STR_LEN]);
 
 void SHA512_Init(SHA512_CTX*);
 void SHA512_Update(SHA512_CTX*, const u_int8_t*, size_t);
-void SHA512_Final(u_int8_t[SHA512_DIGEST_LENGTH], SHA512_CTX*);
-char* SHA512_End(SHA512_CTX*, char[SHA512_DIGEST_STRING_LENGTH]);
-char* SHA512_Data(const u_int8_t*, size_t, char[SHA512_DIGEST_STRING_LENGTH]);
+void SHA512_Final(u_int8_t[SHA512_DIGEST_LEN], SHA512_CTX*);
+char* SHA512_End(SHA512_CTX*, char[SHA512_DIGEST_STR_LEN]);
+char* SHA512_Data(const u_int8_t*, size_t, char[SHA512_DIGEST_STR_LEN]);
 
 #endif /* SHA2_USE_INTTYPES_H */
 
