@@ -61,17 +61,43 @@ FKO_CLIENT_TIMEOUT_LOCAL_NAT_ACCESS_MSG = 6
 
 """Digest type constants
 """
+FKO_DIGEST_INVALID_DATA = -1
+FKO_DIGEST_UNKNOWN = 0
 FKO_DIGEST_MD5 = 1
 FKO_DIGEST_SHA1 = 2
 FKO_DIGEST_SHA256 = 3
 FKO_DIGEST_SHA384 = 4
 FKO_DIGEST_SHA512 = 5
 
+"""Hmac type constants
+"""
+FKO_HMAC_INVALID_DATA = -1
+FKO_HMAC_UNKNOWN = 0
+FKO_HMAC_MD5 = 1
+FKO_HMAC_SHA1 = 2
+FKO_HMAC_SHA256 = 3
+FKO_HMAC_SHA384 = 4
+FKO_HMAC_SHA512 = 5
+
 """Encryption type constants
 """
+FKO_ENCRYPTION_INVALID_DATA = -1
+FKO_ENCRYPTION_UNKNOWN = 0
 FKO_ENCRYPTION_RIJNDAEL = 1
 FKO_ENCRYPTION_GPG = 2
 
+"""Symmetric encryption modes to correspond to rijndael.h
+"""
+FKO_ENC_MODE_UNKNOWN = 0
+FKO_ENC_MODE_ECB = 1
+FKO_ENC_MODE_CBC = 2
+FKO_ENC_MODE_CFB = 3
+FKO_ENC_MODE_PCBC = 4
+FKO_ENC_MODE_OFB = 5
+FKO_ENC_MODE_CTR = 6
+FKO_ENC_MODE_ASYMMETRIC = 7
+FKO_ENC_MODE_CBC_LEGACY_IV = 8
+
 """FKO error codes
 """
 FKO_SUCCESS = 0
@@ -79,49 +105,141 @@ FKO_ERROR_CTX_NOT_INITIALIZED = 1
 FKO_ERROR_MEMORY_ALLOCATION = 2
 FKO_ERROR_FILESYSTEM_OPERATION = 3
 FKO_ERROR_INVALID_DATA = 4
-FKO_ERROR_DATA_TOO_LARGE = 5
-FKO_ERROR_USERNAME_UNKNOWN = 6
-FKO_ERROR_INCOMPLETE_SPA_DATA = 7
-FKO_ERROR_MISSING_ENCODED_DATA = 8
-FKO_ERROR_INVALID_DIGEST_TYPE = 9
-FKO_ERROR_INVALID_ALLOW_IP = 10
-FKO_ERROR_INVALID_SPA_COMMAND_MSG = 11
-FKO_ERROR_INVALID_SPA_ACCESS_MSG = 12
-FKO_ERROR_INVALID_SPA_NAT_ACCESS_MSG = 13
-FKO_ERROR_INVALID_ENCRYPTION_TYPE = 14
-FKO_ERROR_WRONG_ENCRYPTION_TYPE = 15
-FKO_ERROR_DECRYPTION_SIZE = 16
-FKO_ERROR_DECRYPTION_FAILURE = 17
-FKO_ERROR_DIGEST_VERIFICATION_FAILED = 18
-FKO_ERROR_UNSUPPORTED_FEATURE = 19
-FKO_ERROR_UNKNOWN = 20
+FKO_ERROR_INVALID_DATA_CLIENT_TIMEOUT_NEGATIVE = 5
+FKO_ERROR_INVALID_DATA_DECODE_MSGLEN_VALIDFAIL = 6
+FKO_ERROR_INVALID_DATA_DECODE_NON_ASCII = 7
+FKO_ERROR_INVALID_DATA_DECODE_LT_MIN_FIELDS = 8
+FKO_ERROR_INVALID_DATA_DECODE_ENC_MSG_LEN_MT_T_SIZE = 9
+FKO_ERROR_INVALID_DATA_DECODE_RAND_MISSING = 10
+FKO_ERROR_INVALID_DATA_DECODE_USERNAME_MISSING = 11
+FKO_ERROR_INVALID_DATA_DECODE_USERNAME_TOOBIG = 12
+FKO_ERROR_INVALID_DATA_DECODE_USERNAME_DECODEFAIL = 13
+FKO_ERROR_INVALID_DATA_DECODE_USERNAME_VALIDFAIL = 14
+FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_MISSING = 15
+FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_TOOBIG = 16
+FKO_ERROR_INVALID_DATA_DECODE_TIMESTAMP_DECODEFAIL = 17
+FKO_ERROR_INVALID_DATA_DECODE_VERSION_MISSING = 18
+FKO_ERROR_INVALID_DATA_DECODE_VERSION_TOOBIG = 19
+FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_MISSING = 20
+FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_TOOBIG = 21
+FKO_ERROR_INVALID_DATA_DECODE_MSGTYPE_DECODEFAIL = 22
+FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_MISSING = 23
+FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_TOOBIG = 24
+FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_DECODEFAIL = 25
+FKO_ERROR_INVALID_DATA_DECODE_MESSAGE_VALIDFAIL = 26
+FKO_ERROR_INVALID_DATA_DECODE_ACCESS_VALIDFAIL = 27
+FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_MISSING = 28
+FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_TOOBIG = 29
+FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_DECODEFAIL = 30
+FKO_ERROR_INVALID_DATA_DECODE_NATACCESS_VALIDFAIL = 31
+FKO_ERROR_INVALID_DATA_DECODE_SRVAUTH_MISSING = 32
+FKO_ERROR_INVALID_DATA_DECODE_SRVAUTH_DECODEFAIL = 33
+FKO_ERROR_INVALID_DATA_DECODE_SPA_EXTRA_TOOBIG = 34
+FKO_ERROR_INVALID_DATA_DECODE_EXTRA_TOOBIG = 35
+FKO_ERROR_INVALID_DATA_DECODE_EXTRA_DECODEFAIL = 36
+FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_MISSING = 37
+FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_TOOBIG = 38
+FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_VALIDFAIL = 39
+FKO_ERROR_INVALID_DATA_DECODE_TIMEOUT_DECODEFAIL = 40
+FKO_ERROR_INVALID_DATA_ENCODE_MESSAGE_TOOBIG = 41
+FKO_ERROR_INVALID_DATA_ENCODE_MSGLEN_VALIDFAIL = 42
+FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_VALIDFAIL = 43
+FKO_ERROR_INVALID_DATA_ENCODE_DIGEST_TOOBIG = 44
+FKO_ERROR_INVALID_DATA_ENCODE_NOTBASE64 = 45
+FKO_ERROR_INVALID_DATA_ENCRYPT_MSGLEN_VALIDFAIL = 46
+FKO_ERROR_INVALID_DATA_ENCRYPT_DIGESTLEN_VALIDFAIL = 47
+FKO_ERROR_INVALID_DATA_ENCRYPT_PTLEN_VALIDFAIL = 48
+FKO_ERROR_INVALID_DATA_ENCRYPT_RESULT_MSGLEN_VALIDFAIL = 49
+FKO_ERROR_INVALID_DATA_ENCRYPT_CIPHERLEN_DECODEFAIL = 50
+FKO_ERROR_INVALID_DATA_ENCRYPT_CIPHERLEN_VALIDFAIL = 51
+FKO_ERROR_INVALID_DATA_ENCRYPT_DECRYPTED_MESSAGE_MISSING = 52
+FKO_ERROR_INVALID_DATA_ENCRYPT_DECRYPTED_MSGLEN_VALIDFAIL = 53
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_MESSAGE_VALIDFAIL = 54
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_DIGEST_VALIDFAIL = 55
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_MSGLEN_VALIDFAIL = 56
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_RESULT_MSGLEN_VALIDFAIL = 57
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_CIPHER_DECODEFAIL = 58
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_ENCODEDMSG_NULL = 59
+FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_ENCODEDMSGLEN_VALIDFAIL = 60
+FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_VALIDFAIL = 61
+FKO_ERROR_INVALID_DATA_ENCRYPT_MODE_VALIDFAIL = 62
+FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_UNKNOWN = 63
+FKO_ERROR_INVALID_DATA_FUNCS_NEW_ENCMSG_MISSING = 64
+FKO_ERROR_INVALID_DATA_FUNCS_NEW_MSGLEN_VALIDFAIL = 65
+FKO_ERROR_INVALID_DATA_FUNCS_GEN_KEYLEN_VALIDFAIL = 66
+FKO_ERROR_INVALID_DATA_FUNCS_GEN_HMACLEN_VALIDFAIL = 67
+FKO_ERROR_INVALID_DATA_FUNCS_GEN_KEY_ENCODEFAIL = 68
+FKO_ERROR_INVALID_DATA_FUNCS_GEN_HMAC_ENCODEFAIL = 69
+FKO_ERROR_INVALID_DATA_FUNCS_SET_MSGLEN_VALIDFAIL = 70
+FKO_ERROR_INVALID_DATA_HMAC_MSGLEN_VALIDFAIL = 71
+FKO_ERROR_INVALID_DATA_HMAC_ENCMSGLEN_VALIDFAIL = 72
+FKO_ERROR_INVALID_DATA_HMAC_COMPAREFAIL = 73
+FKO_ERROR_INVALID_DATA_HMAC_TYPE_VALIDFAIL = 74
+FKO_ERROR_INVALID_DATA_HMAC_LEN_VALIDFAIL = 75
+FKO_ERROR_INVALID_DATA_MESSAGE_PORT_MISSING = 76
+FKO_ERROR_INVALID_DATA_MESSAGE_TYPE_VALIDFAIL = 77
+FKO_ERROR_INVALID_DATA_MESSAGE_EMPTY = 78
+FKO_ERROR_INVALID_DATA_MESSAGE_CMD_MISSING = 79
+FKO_ERROR_INVALID_DATA_MESSAGE_ACCESS_MISSING = 80
+FKO_ERROR_INVALID_DATA_MESSAGE_NAT_MISSING = 81
+FKO_ERROR_INVALID_DATA_MESSAGE_PORTPROTO_MISSING = 82
+FKO_ERROR_INVALID_DATA_NAT_EMPTY = 83
+FKO_ERROR_INVALID_DATA_RAND_LEN_VALIDFAIL = 84
+FKO_ERROR_INVALID_DATA_SRVAUTH_MISSING = 85
+FKO_ERROR_INVALID_DATA_TIMESTAMP_VALIDFAIL = 86
+FKO_ERROR_INVALID_DATA_USER_MISSING = 87
+FKO_ERROR_INVALID_DATA_USER_FIRSTCHAR_VALIDFAIL = 88
+FKO_ERROR_INVALID_DATA_USER_REMCHAR_VALIDFAIL = 89
+FKO_ERROR_INVALID_DATA_UTIL_STRTOL_LT_MIN = 90
+FKO_ERROR_INVALID_DATA_UTIL_STROL_GT_MAX = 91
+FKO_ERROR_DATA_TOO_LARGE = 92
+FKO_ERROR_INVALID_KEY_LEN = 93
+FKO_ERROR_USERNAME_UNKNOWN = 94
+FKO_ERROR_INCOMPLETE_SPA_DATA = 95
+FKO_ERROR_MISSING_ENCODED_DATA = 96
+FKO_ERROR_INVALID_DIGEST_TYPE = 97
+FKO_ERROR_INVALID_ALLOW_IP = 98
+FKO_ERROR_INVALID_SPA_COMMAND_MSG = 99
+FKO_ERROR_INVALID_SPA_ACCESS_MSG = 100
+FKO_ERROR_INVALID_SPA_NAT_ACCESS_MSG = 101
+FKO_ERROR_INVALID_ENCRYPTION_TYPE = 102
+FKO_ERROR_WRONG_ENCRYPTION_TYPE = 103
+FKO_ERROR_DECRYPTION_SIZE = 104
+FKO_ERROR_DECRYPTION_FAILURE = 105
+FKO_ERROR_DIGEST_VERIFICATION_FAILED = 106
+FKO_ERROR_INVALID_HMAC_KEY_LEN = 107
+FKO_ERROR_UNSUPPORTED_HMAC_MODE = 108
+FKO_ERROR_UNSUPPORTED_FEATURE = 109
+FKO_ERROR_ZERO_OUT_DATA = 110
+FKO_ERROR_UNKNOWN = 111
 # Start GPGME-related errors
-GPGME_ERR_START = 21
-FKO_ERROR_MISSING_GPG_KEY_DATA = 22
-FKO_ERROR_GPGME_NO_OPENPGP = 23
-FKO_ERROR_GPGME_CONTEXT = 24
-FKO_ERROR_GPGME_PLAINTEXT_DATA_OBJ = 25
-FKO_ERROR_GPGME_SET_PROTOCOL = 26
-FKO_ERROR_GPGME_CIPHER_DATA_OBJ = 27
-FKO_ERROR_GPGME_BAD_PASSPHRASE = 28
-FKO_ERROR_GPGME_ENCRYPT_SIGN = 29
-FKO_ERROR_GPGME_CONTEXT_SIGNER_KEY = 30
-FKO_ERROR_GPGME_SIGNER_KEYLIST_START = 31
-FKO_ERROR_GPGME_SIGNER_KEY_NOT_FOUND = 32
-FKO_ERROR_GPGME_SIGNER_KEY_AMBIGUOUS = 33
-FKO_ERROR_GPGME_ADD_SIGNER = 34
-FKO_ERROR_GPGME_CONTEXT_RECIPIENT_KEY = 35
-FKO_ERROR_GPGME_RECIPIENT_KEYLIST_START = 36
-FKO_ERROR_GPGME_RECIPIENT_KEY_NOT_FOUND = 37
-FKO_ERROR_GPGME_RECIPIENT_KEY_AMBIGUOUS = 38
-FKO_ERROR_GPGME_DECRYPT_FAILED = 39
-FKO_ERROR_GPGME_DECRYPT_UNSUPPORTED_ALGORITHM = 40
-FKO_ERROR_GPGME_BAD_GPG_EXE = 41
-FKO_ERROR_GPGME_BAD_HOME_DIR = 42
-FKO_ERROR_GPGME_SET_HOME_DIR = 43
-FKO_ERROR_GPGME_NO_SIGNATURE = 44
-FKO_ERROR_GPGME_BAD_SIGNATURE = 45
-FKO_ERROR_GPGME_SIGNATURE_VERIFY_DISABLED = 46
+GPGME_ERR_START = 112
+FKO_ERROR_MISSING_GPG_KEY_DATA = 113
+FKO_ERROR_GPGME_NO_OPENPGP = 114
+FKO_ERROR_GPGME_CONTEXT = 115
+FKO_ERROR_GPGME_PLAINTEXT_DATA_OBJ = 116
+FKO_ERROR_GPGME_SET_PROTOCOL = 117
+FKO_ERROR_GPGME_CIPHER_DATA_OBJ = 118
+FKO_ERROR_GPGME_BAD_PASSPHRASE = 119
+FKO_ERROR_GPGME_ENCRYPT_SIGN = 120
+FKO_ERROR_GPGME_CONTEXT_SIGNER_KEY = 121
+FKO_ERROR_GPGME_SIGNER_KEYLIST_START = 122
+FKO_ERROR_GPGME_SIGNER_KEY_NOT_FOUND = 123
+FKO_ERROR_GPGME_SIGNER_KEY_AMBIGUOUS = 124
+FKO_ERROR_GPGME_ADD_SIGNER = 125
+FKO_ERROR_GPGME_CONTEXT_RECIPIENT_KEY = 126
+FKO_ERROR_GPGME_RECIPIENT_KEYLIST_START = 127
+FKO_ERROR_GPGME_RECIPIENT_KEY_NOT_FOUND = 128
+FKO_ERROR_GPGME_RECIPIENT_KEY_AMBIGUOUS = 129
+FKO_ERROR_GPGME_DECRYPT_FAILED = 130
+FKO_ERROR_GPGME_DECRYPT_UNSUPPORTED_ALGORITHM = 131
+FKO_ERROR_GPGME_BAD_GPG_EXE = 132
+FKO_ERROR_GPGME_BAD_HOME_DIR = 133
+FKO_ERROR_GPGME_SET_HOME_DIR = 134
+FKO_ERROR_GPGME_NO_SIGNATURE = 135
+FKO_ERROR_GPGME_BAD_SIGNATURE = 136
+FKO_ERROR_GPGME_SIGNATURE_VERIFY_DISABLED = 137
+
 
 ### End FKO Constants ###
 
@@ -423,7 +541,51 @@ class Fko:
         """
         return _fko.get_encoded_data(self.ctx)
 
-    def spa_data_final(self, key):
+    def raw_spa_digest_type(self, val=None):
+        """Get or set the raw spa_digest_type
+
+        This is an integer value. If no argument is given, the current value
+        is returned.  Otherwise, the SPA message client timeout value will
+        be set to the given value.
+        """
+        if val != None:
+            _fko.set_raw_spa_digest_type(self.ctx, val)
+        else:
+            return _fko.get_raw_spa_digest_type(self.ctx)
+
+    def raw_spa_digest(self, val=None):
+        """Get or set the raw spa_digest_type
+        """
+        if val != None:
+            _fko.set_raw_spa_digest(self.ctx, val)
+        else:
+            return _fko.get_raw_spa_digest(self.ctx)
+
+    def spa_encryption_mode(self, val=None):
+        """Get or set the spa_encryption mode
+
+        This is an integer value. If no argument is given, the current value
+        is returned.  Otherwise, the SPA message client timeout value will
+        be set to the given value.
+        """
+        if val != None:
+            _fko.set_spa_encryption_mode(self.ctx, val)
+        else:
+            return _fko.get_spa_encryption_mode(self.ctx)
+
+    def hmac_type(self, val=None):
+        """Get or set the spa_hmac_type
+
+        This is an integer value. If no argument is given, the current value
+        is returned.  Otherwise, the SPA message client timeout value will
+        be set to the given value.
+        """
+        if val != None:
+            _fko.set_spa_hmac_type(self.ctx, val)
+        else:
+            return _fko.get_spa_hmac_type(self.ctx)
+
+    def spa_data_final(self, key, hmac_key):
         """Perform final processing and generation of the SPA message data.
 
         This function is the final step in creating a complete encrypted
@@ -431,7 +593,7 @@ class Fko:
         does require all of the requisite SPA data fields be set. Otherwise,
         it will fail and throw an fko.error exception.
         """
-        _fko.spa_data_final(self.ctx, key)
+        _fko.spa_data_final(self.ctx, key, hmac_key)
 
     def gen_spa_data(self, key):
         """Alias for "spa_data_final()".
@@ -493,6 +655,44 @@ class Fko:
         """
         _fko.decrypt_spa_data(self.ctx, key)
 
+# --DSS
+
+    def encryption_type(self, enc_data):
+        """Return the assumed encryption type based on the encryptped data
+        """
+        _fko.encryption_type(enc_data)
+
+    def key_gen(self, keyb64, hmac_keyb64):
+        """Generate Rijndael and HMAC keys and base64 encode them
+        """
+        _fko.key_gen(keyb64, hmac_keyb64)
+
+    def base64_encode(self, indata):
+        """Base64 encode function
+        """
+        _fko.base64_encode(indata)
+
+    def base64_decode(self, indata):
+        """Base64 decode function
+        """
+        _fko.base64_decode(indata)
+
+    def verify_hmac(self, hmac_key):
+        """Generate HMAC for the data and verify it against the HMAC included with the data
+        """
+        _fko.verify_hmac(self.ctx, hmac_key)
+
+    def set_spa_hmac(self, hmac_key):
+        """Calculate the HMAC for the given data
+        """
+        _fko.set_spa_hmac(self.ctx, hmac_key)
+
+    def get_spa_hmac(self):
+        """Return the HMAC for the data in the current context
+        """
+        _fko.get_spa_hmac(self.ctx)
+
+
     # GPG-related functions.
 
     def gpg_recipient(self, val=None):
@@ -637,6 +837,11 @@ class Fko:
             return True
         return False
 
+    def gpg_errstr(self):
+        """Return the last GPG-related error on the current context
+        """
+        _fko.fko_gpg_errstr(self.ctx)
+
     # Error message string function.
 
     def errstr(self, val):
@@ -668,16 +873,23 @@ class Fko:
         elif val == FKO_CLIENT_TIMEOUT_LOCAL_NAT_ACCESS_MSG:
             mts = "Local NAT Access Message with timeout"
         else:
-            mts = "Unknown message type"
+            mts = "Unknown SPA message type"
         return mts
 
     def digest_type_str(self, val=None):
         """Returns the digest type string for the given value.
+
+        If no value is given, the digest type for the current context
+        is returned.
         """
         if val == None:
             val = _fko.get_spa_digest_type(self.ctx)
 
-        if val == FKO_DIGEST_MD5:
+        if val == FKO_DIGEST_INVALID_DATA:
+            dts = "invalid_data"
+        elif val == FKO_DIGEST_UNKNOWN:
+            dts = "unknown"
+        elif val == FKO_DIGEST_MD5:
             dts = "MD5"
         elif val == FKO_DIGEST_SHA1:
             dts = "SHA1"
@@ -688,16 +900,50 @@ class Fko:
         elif val == FKO_DIGEST_SHA512:
             dts = "SHA512"
         else:
-            dts = "Unknown digest type"
+            dts = "Invalid digest type value"
         return dts
 
+    def hmac_type_str(self, val=None):
+        """Returns the HMAC type string for the given value.
+
+        If no value is given, the HMAC type for the current context
+        is returned.
+        """
+        if val == None:
+            val = _fko.get_spa_hmac_type(self.ctx)
+
+        if val == FKO_HMAC_INVALID_DATA:
+            ht = "invalid_data"
+        elif val == FKO_HMAC_UNKNOWN:
+            ht = "unknown"
+        elif val == FKO_HMAC_MD5:
+            ht = "MD5"
+        elif val == FKO_HMAC_SHA1:
+            ht = "SHA1"
+        elif val == FKO_HMAC_SHA256:
+            ht = "SHA256"
+        elif val == FKO_HMAC_SHA384:
+            ht = "SHA384"
+        elif val == FKO_HMAC_SHA512:
+            ht = "SHA512"
+        else:
+            ht = "Invalid HMAC digest type value"
+        return ht
+
     def encryption_type_str(self, val=None):
         """Returns the encryption type string for the given value.
+
+        If no value is given, the encryption type for the current context
+        is returned.
         """
         if val == None:
             val = _fko.get_spa_encryption_type(self.ctx)
 
-        if val == FKO_ENCRYPTION_RIJNDAEL:
+        if val == FKO_ENCRYPTION_INVALID_DATA:
+            ets = "invalid_data"
+        elif val == FKO_ENCRYPTION_UNKNOWN:
+            ets = "unknown"
+        elif val == FKO_ENCRYPTION_RIJNDAEL:
             ets = "Rijndael (AES)"
         elif val == FKO_ENCRYPTION_GPG:
             ets = "GPG"
@@ -705,6 +951,38 @@ class Fko:
             ets = "Unknown encryption type"
         return ets
 
+    def encryption_mode_str(self, val=None):
+        """Returns the encryption mode string for the given value.
+
+        If no value is given, the encryption mode for the current context
+        is returned.
+        """
+        if val == None:
+            val = _fko.get_spa_encryption_mode(self.ctx)
+
+        if val == FKO_ENC_MODE_UNKNOWN:
+            dts = "unknown"
+        elif val == FKO_ENC_MODE_ECB:
+            dts = "ECB"
+        elif val == FKO_ENC_MODE_CBC:
+            dts = "CBC"
+        elif val == FKO_ENC_MODE_CBF:
+            dts = "CBF"
+        elif val == FKO_ENC_MODE_PCBC:
+            dts = "PCBC"
+        elif val == FKO_ENC_MODE_OFB:
+            dts = "OFB"
+        elif val == FKO_ENC_MODE_CTR:
+            dts = "CTR"
+        elif val == FKO_ENC_MODE_ASYMMETRIC:
+            dts = "ASYMMETRIC"
+        elif val == FKO_ENC_MODE_CBC_LEGACY_IV:
+            dts = "CBC_LEGACY_IV"
+        else:
+            dts = "Invalid encryption mode value"
+        return dts
+
+
     def __call__(self):
         """Calls the spa_data() method.
 
