@@ -35,6 +35,10 @@
 #include "utils.h"
 #include "log_msg.h"
 
+#if FIREWALL_IPTABLES
+  #include "fw_util_iptables.h"
+#endif
+
 /* Check to see if an integer variable has a value that is within a
  * specific range
 */
@@ -440,6 +444,24 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_ENABLE_IPT_SNAT,
             DEF_ENABLE_IPT_SNAT);
 
+    /* Make sure we have a valid IP if SNAT is enabled
+    */
+    if(strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1) == 0)
+    {
+        /* Note that fw_config_init() will set use_masquerade if necessary
+        */
+        if(opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
+        {
+            if(! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
+            {
+                log_msg(LOG_ERR,
+                    "Invalid IPv4 addr for SNAT_TRANSLATE_IP"
+                );
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+            }
+        }
+    }
+
     /* Enable IPT OUTPUT.
     */
     if(opts->config[CONF_ENABLE_IPT_OUTPUT] == NULL)
@@ -462,36 +484,84 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_IPT_INPUT_ACCESS,
             DEF_IPT_INPUT_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_INPUT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_INPUT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* IPT output access.
     */
     if(opts->config[CONF_IPT_OUTPUT_ACCESS] == NULL)
         set_config_entry(opts, CONF_IPT_OUTPUT_ACCESS,
             DEF_IPT_OUTPUT_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_OUTPUT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_OUTPUT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* IPT forward access.
     */
     if(opts->config[CONF_IPT_FORWARD_ACCESS] == NULL)
         set_config_entry(opts, CONF_IPT_FORWARD_ACCESS,
             DEF_IPT_FORWARD_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_FORWARD_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_FORWARD_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* IPT dnat access.
     */
     if(opts->config[CONF_IPT_DNAT_ACCESS] == NULL)
         set_config_entry(opts, CONF_IPT_DNAT_ACCESS,
             DEF_IPT_DNAT_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_DNAT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_DNAT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* IPT snat access.
     */
     if(opts->config[CONF_IPT_SNAT_ACCESS] == NULL)
         set_config_entry(opts, CONF_IPT_SNAT_ACCESS,
             DEF_IPT_SNAT_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_SNAT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_SNAT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* IPT masquerade access.
     */
     if(opts->config[CONF_IPT_MASQUERADE_ACCESS] == NULL)
         set_config_entry(opts, CONF_IPT_MASQUERADE_ACCESS,
             DEF_IPT_MASQUERADE_ACCESS);
 
+    if(validate_ipt_chain_conf(opts->config[CONF_IPT_MASQUERADE_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid IPT_MASQUERADE_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
     /* Check for the iptables 'comment' match at init time
     */
     if(opts->config[CONF_ENABLE_IPT_COMMENT_CHECK] == NULL)
@@ -871,6 +941,9 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'v':
                 opts->verbose++;
                 break;
+            case SYSLOG_ENABLE:
+                opts->syslog_enable = 1;
+                break;
             case 'V':
                 fprintf(stdout, "fwknopd server %s\n", MY_VERSION);
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
@@ -944,6 +1017,8 @@ usage(void)
       "                           '<name>-old', and starting a new one.\n"
       " -S, --status            - Display the status of any running fwknopd process.\n"
       " -v, --verbose           - Set verbose mode.\n"
+      "     --syslog-enable     - Allow messages to be sent to syslog even if the\n"
+      "                           foreground mode is set.\n"
       " -V, --version           - Print version number.\n"
       "     --fw-list           - List all firewall rules that fwknop has created\n"
       "                           and then exit.\n"
