@@ -7,9 +7,9 @@
  *
  * Purpose: Fwknop routines for managing ipf firewall rules.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -71,14 +71,14 @@ fw_config_init(fko_srv_options_t *opts)
 
     /* Set our firewall exe command path (iptables in most cases).
     */
-    strlcpy(fwc.fw_command, opts->config[CONF_FIREWALL_EXE], MAX_PATH_LEN);
+    strlcpy(fwc.fw_command, opts->config[CONF_FIREWALL_EXE], sizeof(fwc.fw_command));
 
 
     /* Let us find it via our opts struct as well.
     */
     opts->fw_config = &fwc;
 
-    return;
+    return 1;
 }
 
 void
@@ -90,9 +90,11 @@ fw_initialize(const fko_srv_options_t *opts)
 
     if(res != 0)
     {
-        fprintf(stderr, "Warning: Errors detected during fwknop custom chain creation.\n");
-        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+        log_msg(LOG_WARNING,
+                "Warning: Errors detected during fw_initialize().");
+        return 0;
     }
+    return 1;
 }
 
 int
@@ -121,9 +123,6 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
     acc_port_list_t *port_list = NULL;
     acc_port_list_t *ple;
 
-    unsigned int    fst_proto;
-    unsigned int    fst_port;
-
     int             res = 0;
     time_t          now;
     unsigned int    exp_ts;
@@ -136,12 +135,6 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
     */
     ple = port_list;
 
-    /* Remember the first proto/port combo in case we need them
-     * for NAT access requests.
-    */
-    fst_proto = ple->proto;
-    fst_port  = ple->port;
-
     /* Set our expire time value.
     */
     time(&now);
@@ -161,8 +154,8 @@ check_firewall_rules(const fko_srv_options_t *opts)
 
     /* TODO: Implement me */
 
-    char             exp_str[12];
-    char             rule_num_str[6];
+    char             exp_str[12]     = {0};
+    char             rule_num_str[6] = {0};
     char            *ndx, *rn_start, *rn_end, *tmp_mark;
 
     int             i, res, rn_offset;
