@@ -7,9 +7,9 @@
  *
  * Purpose: Fwknop routines for managing ipfw firewall rules.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -80,9 +80,8 @@ ipfw_set_exists(const fko_srv_options_t *opts,
 
     res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
 
-    if (opts->verbose)
-        log_msg(LOG_INFO, "ipfw_set_exists() CMD: '%s' (res: %d)",
-            cmd_buf, res);
+    log_msg(LOG_DEBUG, "ipfw_set_exists() CMD: '%s' (res: %d)",
+        cmd_buf, res);
 
     if(!EXTCMD_IS_SUCCESS(res))
         return(0);
@@ -97,7 +96,7 @@ ipfw_set_exists(const fko_srv_options_t *opts,
  * daemon to stdout.
 */
 int
-fw_dump_rules(const fko_srv_options_t *opts)
+fw_dump_rules(const fko_srv_options_t * const opts)
 {
     int     res, got_err = 0;
 
@@ -116,9 +115,8 @@ fw_dump_rules(const fko_srv_options_t *opts)
 
         res = system(cmd_buf);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_dump_rules() CMD: '%s' (res: %d)",
-                cmd_buf, res);
+        log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
+            cmd_buf, res);
 
         /* Expect full success on this */
         if(! EXTCMD_IS_SUCCESS(res))
@@ -144,9 +142,8 @@ fw_dump_rules(const fko_srv_options_t *opts)
         printf("\nActive Rules:\n");
         res = system(cmd_buf);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_dump_rules() CMD: '%s' (res: %d)",
-                cmd_buf, res);
+        log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
+            cmd_buf, res);
 
         /* Expect full success on this */
         if(! EXTCMD_IS_SUCCESS(res))
@@ -165,9 +162,8 @@ fw_dump_rules(const fko_srv_options_t *opts)
         printf("\nExpired Rules:\n");
         res = system(cmd_buf);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_dump_rules() CMD: '%s' (res: %d)",
-                cmd_buf, res);
+        log_msg(LOG_DEBUG, "fw_dump_rules() CMD: '%s' (res: %d)",
+            cmd_buf, res);
 
         /* Expect full success on this */
         if(! EXTCMD_IS_SUCCESS(res))
@@ -180,33 +176,74 @@ fw_dump_rules(const fko_srv_options_t *opts)
     return(got_err);
 }
 
-void
-fw_config_init(fko_srv_options_t *opts)
+int
+fw_config_init(fko_srv_options_t * const opts)
 {
+    int         is_err;
 
     memset(&fwc, 0x0, sizeof(struct fw_config));
 
     /* Set our firewall exe command path (iptables in most cases).
     */
-    strlcpy(fwc.fw_command, opts->config[CONF_FIREWALL_EXE], MAX_PATH_LEN);
+    strlcpy(fwc.fw_command, opts->config[CONF_FIREWALL_EXE], sizeof(fwc.fw_command));
 
-    fwc.start_rule_num = atoi(opts->config[CONF_IPFW_START_RULE_NUM]);
-    fwc.max_rules      = atoi(opts->config[CONF_IPFW_MAX_RULES]);
-    fwc.active_set_num = atoi(opts->config[CONF_IPFW_ACTIVE_SET_NUM]);
-    fwc.expire_set_num = atoi(opts->config[CONF_IPFW_EXPIRE_SET_NUM]);
-    fwc.purge_interval = atoi(opts->config[CONF_IPFW_EXPIRE_PURGE_INTERVAL]);
+    fwc.start_rule_num = strtol_wrapper(opts->config[CONF_IPFW_START_RULE_NUM],
+            0, RCHK_MAX_IPFW_MAX_RULES, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] IPFW_START_RULE_NUM '%s' out of range [%d-%d].",
+                opts->config[CONF_IPFW_START_RULE_NUM], 0, RCHK_MAX_IPFW_MAX_RULES);
+        return 0;
+    }
+
+    fwc.max_rules = strtol_wrapper(opts->config[CONF_IPFW_MAX_RULES],
+            0, RCHK_MAX_IPFW_MAX_RULES, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] IPFW_MAX_RULES_INT '%s' out of range [%d-%d].",
+                opts->config[CONF_IPFW_MAX_RULES], 0, RCHK_MAX_IPFW_MAX_RULES);
+        return 0;
+    }
+
+    fwc.active_set_num = strtol_wrapper(opts->config[CONF_IPFW_ACTIVE_SET_NUM],
+            0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] IPFW_ACTIVE_SET_NUM '%s' out of range [%d-%d].",
+                opts->config[CONF_IPFW_ACTIVE_SET_NUM], 0, RCHK_MAX_IPFW_SET_NUM);
+        return 0;
+    }
+
+    fwc.expire_set_num = strtol_wrapper(opts->config[CONF_IPFW_EXPIRE_SET_NUM],
+            0, RCHK_MAX_IPFW_SET_NUM, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] IPFW_MAX_EXPIRE_SET_NUM '%s' out of range [%d-%d].",
+                opts->config[CONF_IPFW_EXPIRE_SET_NUM], 0, RCHK_MAX_IPFW_SET_NUM);
+        return 0;
+    }
+
+    fwc.purge_interval = strtol_wrapper(opts->config[CONF_IPFW_EXPIRE_PURGE_INTERVAL],
+            0, RCHK_MAX_IPFW_PURGE_INTERVAL, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] IPFW_EXPIRE_PURGE_INTERVAL '%s' out of range [%d-%d].",
+                opts->config[CONF_IPFW_EXPIRE_PURGE_INTERVAL], 0,
+                RCHK_MAX_IPFW_PURGE_INTERVAL);
+        return 0;
+    }
 
     /* Let us find it via our opts struct as well.
     */
     opts->fw_config = &fwc;
 
-    return;
+    return 1;
 }
 
-void
-fw_initialize(const fko_srv_options_t *opts)
+int
+fw_initialize(const fko_srv_options_t * const opts)
 {
-    int             res = 0;
+    int             res = 0, is_err;
     unsigned short  curr_rule;
     char           *ndx;
 
@@ -217,8 +254,8 @@ fw_initialize(const fko_srv_options_t *opts)
 
     if(res != 0)
     {
-        fprintf(stderr, "Fatal: Errors detected during ipfw rules initialization.\n");
-        exit(EXIT_FAILURE);
+        log_msg(LOG_ERR, "[*] Fatal: Errors detected during ipfw rules initialization.");
+        return 0;
     }
 
     /* Allocate our rule_map array for tracking active (and expired) rules.
@@ -227,8 +264,8 @@ fw_initialize(const fko_srv_options_t *opts)
 
     if(fwc.rule_map == NULL)
     {
-        fprintf(stderr, "Fatal: Memory allocation error in fw_initialize.\n");
-        exit(EXIT_FAILURE);
+        log_msg(LOG_ERR, "[*] Fatal: Memory allocation error in fw_initialize().");
+        return 0;
     }
 
     /* Create a check-state rule if necessary.
@@ -245,9 +282,8 @@ fw_initialize(const fko_srv_options_t *opts)
 
         res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
-                cmd_buf, res, err_buf);
+        log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
+            cmd_buf, res, err_buf);
 
         if(EXTCMD_IS_SUCCESS(res))
         {
@@ -276,9 +312,8 @@ fw_initialize(const fko_srv_options_t *opts)
 
         res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
-                cmd_buf, res, err_buf);
+        log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d, err: %s)",
+            cmd_buf, res, err_buf);
 
         if(EXTCMD_IS_SUCCESS(res))
             log_msg(LOG_INFO, "Set ipfw expire set %u to disabled.",
@@ -299,18 +334,16 @@ fw_initialize(const fko_srv_options_t *opts)
 
     res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
 
-    if (opts->verbose)
-        log_msg(LOG_INFO, "fw_initialize() CMD: '%s' (res: %d)",
-            cmd_buf, res);
+    log_msg(LOG_DEBUG, "fw_initialize() CMD: '%s' (res: %d)",
+        cmd_buf, res);
 
     if(!EXTCMD_IS_SUCCESS(res))
     {
         log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, cmd_out);
-        return;
+        return 0;
     }
 
-    if(opts->verbose > 1)
-        log_msg(LOG_INFO, "RULES LIST: %s", cmd_out);
+    log_msg(LOG_DEBUG, "RULES LIST: %s", cmd_out);
 
     /* Find the first "# DISABLED" string (if any).
     */
@@ -319,7 +352,7 @@ fw_initialize(const fko_srv_options_t *opts)
     /* Assume no disabled rules if we did not see the string.
     */
     if(ndx == NULL)
-        return;
+        return 1;
 
     /* Otherwise we walk each line to pull the rule number and
      * set the appropriate rule map entries.
@@ -332,27 +365,32 @@ fw_initialize(const fko_srv_options_t *opts)
 
         if(isdigit(*ndx))
         {
-            curr_rule = atoi(ndx);
+            curr_rule = strtol_wrapper(ndx, 0, -1, NO_EXIT_UPON_ERR, &is_err);
 
-            if(curr_rule >= fwc.start_rule_num
-              && curr_rule < fwc.start_rule_num + fwc.max_rules)
+            if(is_err == FKO_SUCCESS)
             {
-                fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_EXPIRED;
-                fwc.total_rules++;
+                if(curr_rule >= fwc.start_rule_num
+                  && curr_rule < fwc.start_rule_num + fwc.max_rules)
+                {
+                    fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_EXPIRED;
+                    fwc.total_rules++;
+                }
             }
         }
         else
-            log_msg(LOG_WARNING, "fw_initialize: No rule number found where expected.");
+            log_msg(LOG_WARNING,
+                    "fw_initialize: No rule number found where expected.");
 
         /* Find the next "# DISABLED" string (if any).
         */
         ndx = strstr(ndx, "# DISABLED ");
     }
+
+    return 1;
 }
 
-
 int
-fw_cleanup(const fko_srv_options_t *opts)
+fw_cleanup(const fko_srv_options_t * const opts)
 {
     int     res, got_err = 0;
 
@@ -377,14 +415,13 @@ fw_cleanup(const fko_srv_options_t *opts)
 
         res = system(cmd_buf);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "fw_cleanup() CMD: '%s' (res: %d)",
-                cmd_buf, res);
+        log_msg(LOG_DEBUG, "fw_cleanup() CMD: '%s' (res: %d)",
+            cmd_buf, res);
 
         /* Expect full success on this */
         if(! EXTCMD_IS_SUCCESS(res))
         {
-            log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf); 
+            log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
             got_err++;
         }
     }
@@ -427,16 +464,14 @@ fw_cleanup(const fko_srv_options_t *opts)
 /* Rule Processing - Create an access request...
 */
 int
-process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_data_t *spadat)
+process_spa_request(const fko_srv_options_t * const opts,
+        const acc_stanza_t * const acc, spa_data_t * const spadat)
 {
     unsigned short   rule_num;
 
     acc_port_list_t *port_list = NULL;
     acc_port_list_t *ple;
 
-    unsigned int    fst_proto;
-    unsigned int    fst_port;
-
     int             res = 0;
     time_t          now;
     unsigned int    exp_ts;
@@ -449,12 +484,6 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
     */
     ple = port_list;
 
-    /* Remember the first proto/port combo in case we need them
-     * for NAT access requests.
-    */
-    fst_proto = ple->proto;
-    fst_port  = ple->port;
-
     /* Set our expire time value.
     */
     time(&now);
@@ -475,7 +504,9 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
         */
         if(rule_num == 0)
         {
-            log_msg(LOG_WARNING, "Access request rejected: Maximum allowed number of rules has been reached.");
+            log_msg(LOG_WARNING,
+                    "Access request rejected: Maximum allowed number of rules has been reached.");
+            free_acc_port_list(port_list);
             return(-1);
         }
 
@@ -497,9 +528,8 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
 
             res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
 
-            if (opts->verbose)
-                log_msg(LOG_INFO, "process_spa_request() CMD: '%s' (res: %d, err: %s)",
-                    cmd_buf, res, err_buf);
+            log_msg(LOG_DEBUG, "process_spa_request() CMD: '%s' (res: %d, err: %s)",
+                cmd_buf, res, err_buf);
 
             if(EXTCMD_IS_SUCCESS(res))
             {
@@ -521,7 +551,7 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
                     fwc.next_expire = exp_ts;
             }
             else
-                log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf); 
+                log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
 
             ple = ple->next;
         }
@@ -542,9 +572,11 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
             log_msg(LOG_WARNING, "Forwarding/NAT requests are not currently supported.");
         }
 
+        free_acc_port_list(port_list);
         return(-1);
     }
 
+    free_acc_port_list(port_list);
     return(res);
 }
 
@@ -552,13 +584,13 @@ process_spa_request(const fko_srv_options_t *opts, const acc_stanza_t *acc, spa_
  * firewall rules.
 */
 void
-check_firewall_rules(const fko_srv_options_t *opts)
+check_firewall_rules(const fko_srv_options_t * const opts)
 {
-    char            exp_str[12];
-    char            rule_num_str[6];
+    char            exp_str[12]     = {0};
+    char            rule_num_str[6] = {0};
     char           *ndx, *rn_start, *rn_end, *tmp_mark;
 
-    int             i=0, res=0;
+    int             i=0, res=0, is_err;
     time_t          now, rule_exp, min_exp = 0;
     unsigned short  curr_rule;
 
@@ -590,9 +622,8 @@ check_firewall_rules(const fko_srv_options_t *opts)
 
     res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
 
-    if (opts->verbose)
-        log_msg(LOG_INFO, "check_firewall_rules() CMD: '%s' (res: %d)",
-            cmd_buf, res);
+    log_msg(LOG_DEBUG, "check_firewall_rules() CMD: '%s' (res: %d)",
+        cmd_buf, res);
 
     if(!EXTCMD_IS_SUCCESS(res))
     {
@@ -600,8 +631,7 @@ check_firewall_rules(const fko_srv_options_t *opts)
         return;
     }
 
-    if(opts->verbose > 1)
-        log_msg(LOG_INFO, "RULES LIST: %s", cmd_out);
+    log_msg(LOG_DEBUG, "RULES LIST: %s", cmd_out);
 
     /* Find the first _exp_ string (if any).
     */
@@ -612,7 +642,7 @@ check_firewall_rules(const fko_srv_options_t *opts)
         /* we did not find an expected rule.
         */
         log_msg(LOG_ERR,
-            "Did not find expire comment in rules list %i.\n", i);
+            "Did not find expire comment in rules list %i.", i);
 
         if (fwc.active_rules > 0)
             fwc.active_rules--;
@@ -632,7 +662,7 @@ check_firewall_rules(const fko_srv_options_t *opts)
         */
         tmp_mark = ndx;
 
-        strlcpy(exp_str, ndx, 11);
+        strlcpy(exp_str, ndx, sizeof(exp_str));
         rule_exp = (time_t)atoll(exp_str);
 
         if(rule_exp <= now)
@@ -682,37 +712,43 @@ check_firewall_rules(const fko_srv_options_t *opts)
 
             strlcpy(rule_num_str, rn_start, (rn_end - rn_start)+1);
 
-            curr_rule = atoi(rule_num_str);
+            curr_rule = strtol_wrapper(rule_num_str, 0, -1, NO_EXIT_UPON_ERR, &is_err);
 
-            zero_cmd_buffers();
-
-            /* Move the rule to the expired rules set.
-            */
-            snprintf(cmd_buf, CMD_BUFSIZE-1, "%s " IPFW_MOVE_RULE_ARGS,
-                opts->fw_config->fw_command,
-                curr_rule,
-                fwc.expire_set_num
-            );
-
-            res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
-
-            if (opts->verbose)
-                log_msg(LOG_INFO, "check_firewall_rules() CMD: '%s' (res: %d, err: %s)",
-                    cmd_buf, res, err_buf);
-
-            if(EXTCMD_IS_SUCCESS(res))
+            if(is_err == FKO_SUCCESS)
             {
-                log_msg(LOG_INFO, "Moved rule %s with expire time of %u to set %u.",
-                    rule_num_str, rule_exp, fwc.expire_set_num
+                zero_cmd_buffers();
+
+                /* Move the rule to the expired rules set.
+                */
+                snprintf(cmd_buf, CMD_BUFSIZE-1, "%s " IPFW_MOVE_RULE_ARGS,
+                    opts->fw_config->fw_command,
+                    curr_rule,
+                    fwc.expire_set_num
                 );
 
-                if (fwc.active_rules > 0)
-                    fwc.active_rules--;
+                res = run_extcmd(cmd_buf, err_buf, CMD_BUFSIZE, 0);
 
-                fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_EXPIRED;
+                log_msg(LOG_DEBUG, "check_firewall_rules() CMD: '%s' (res: %d, err: %s)",
+                    cmd_buf, res, err_buf);
+
+                if(EXTCMD_IS_SUCCESS(res))
+                {
+                    log_msg(LOG_INFO, "Moved rule %s with expire time of %u to set %u.",
+                        rule_num_str, rule_exp, fwc.expire_set_num
+                    );
+
+                    if (fwc.active_rules > 0)
+                        fwc.active_rules--;
+
+                    fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_EXPIRED;
+                }
+                else
+                    log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
             }
             else
+            {
                 log_msg(LOG_ERR, "Error %i from cmd:'%s': %s", res, cmd_buf, err_buf);
+            }
         }
         else
         {
@@ -744,9 +780,7 @@ void
 ipfw_purge_expired_rules(const fko_srv_options_t *opts)
 {
     char           *ndx, *co_end;
-
-    int             i, res;
-
+    int             i, res, is_err;
     unsigned short  curr_rule;
 
     /* First, we get the current active dynamic rules for the expired rule
@@ -762,9 +796,8 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
 
     res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
 
-    if (opts->verbose)
-        log_msg(LOG_INFO, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
-            cmd_buf, res);
+    log_msg(LOG_DEBUG, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
+        cmd_buf, res);
 
     if(!EXTCMD_IS_SUCCESS(res))
     {
@@ -779,8 +812,7 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
     {
         co_end = cmd_out + strlen(cmd_out);
 
-        if(opts->verbose > 1)
-            log_msg(LOG_INFO, "EXP RULES LIST: %s", cmd_out);
+        log_msg(LOG_DEBUG, "EXP RULES LIST: %s", cmd_out);
 
         /* Find the "## Dynamic rules" string.
         */
@@ -824,11 +856,14 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
             */
             if(isdigit(*ndx))
             {
-                curr_rule = atoi(ndx);
+                curr_rule = strtol_wrapper(ndx, 0, -1, NO_EXIT_UPON_ERR, &is_err);
 
-                if(curr_rule >= fwc.start_rule_num
-                  && curr_rule < fwc.start_rule_num + fwc.max_rules)
-                    fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_TMP_MARKED;
+                if(is_err == FKO_SUCCESS)
+                {
+                    if(curr_rule >= fwc.start_rule_num
+                      && curr_rule < fwc.start_rule_num + fwc.max_rules)
+                        fwc.rule_map[curr_rule - fwc.start_rule_num] = RULE_TMP_MARKED;
+                }
             }
 
             ndx = strchr(ndx, '\n');
@@ -868,9 +903,8 @@ ipfw_purge_expired_rules(const fko_srv_options_t *opts)
 
         res = run_extcmd(cmd_buf, cmd_out, STANDARD_CMD_OUT_BUFSIZE, 0);
 
-        if (opts->verbose)
-            log_msg(LOG_INFO, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
-                cmd_buf, res);
+        log_msg(LOG_DEBUG, "ipfw_purge_expired_rules() CMD: '%s' (res: %d)",
+            cmd_buf, res);
 
         if(!EXTCMD_IS_SUCCESS(res))
         {
