@@ -728,6 +728,17 @@ set_fw_chain_conf(const int type, const char * const conf_str)
             else
                 tbuf[i++] = *ndx;
         }
+        if(*ndx != '\0'
+                && *ndx != ' '
+                && *ndx != ','
+                && *ndx != '_'
+                && isalnum(*ndx) == 0)
+        {
+            log_msg(LOG_ERR, "[*] Custom chain config parse error: "
+                "invalid character '%c' for chain type %i, "
+                "line: %s", *ndx, type, conf_str);
+            return 0;
+        }
         ndx++;
     }
 
@@ -736,9 +747,9 @@ set_fw_chain_conf(const int type, const char * const conf_str)
     */
     if(j != FW_NUM_CHAIN_FIELDS)
     {
-        log_msg(LOG_ERR, "[*] Custom Chain config parse error.\n"
-            "Wrong number of fields for chain type %i\n"
-            "Line: %s", type, conf_str);
+        log_msg(LOG_ERR, "[*] Custom chain config parse error: "
+            "wrong number of fields for chain type %i, "
+            "line: %s", type, conf_str);
         return 0;
     }
 
@@ -803,7 +814,6 @@ fw_config_init(fko_srv_options_t * const opts)
     */
     if(strncasecmp(opts->config[CONF_ENABLE_IPT_FORWARDING], "Y", 1)==0)
     {
-
         if(set_fw_chain_conf(IPT_FORWARD_ACCESS, opts->config[CONF_IPT_FORWARD_ACCESS]) != 1)
             return 0;
 
@@ -814,24 +824,24 @@ fw_config_init(fko_srv_options_t * const opts)
         */
         if(strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1)==0)
         {
-            /* If an SNAT_TRANSLATE_IP is specified use the SNAT_ACCESS mode.
-             * Otherwise, use MASQUERADE_ACCESS.
-             *
-             * XXX: --DSS: Not sure if using the TRANSLATE_IP parameter as
-             *             the determining factor is the best why to handle
-             *             this.
-             *
-            */
-            if((opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
-              && (strncasecmp(opts->config[CONF_SNAT_TRANSLATE_IP], "__CHANGEME__", 10)) != 0)
+            if(opts->config[CONF_SNAT_TRANSLATE_IP] == NULL
+                    || ! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
             {
-                if(set_fw_chain_conf(IPT_SNAT_ACCESS, opts->config[CONF_IPT_SNAT_ACCESS]) != 1)
+                fwc.use_masquerade = 1;
+                if(set_fw_chain_conf(IPT_MASQUERADE_ACCESS, opts->config[CONF_IPT_MASQUERADE_ACCESS]) != 1)
                     return 0;
             }
             else
             {
-                if(set_fw_chain_conf(IPT_MASQUERADE_ACCESS, opts->config[CONF_IPT_MASQUERADE_ACCESS]) != 1)
+                if(is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
+                {
+                    if(set_fw_chain_conf(IPT_SNAT_ACCESS, opts->config[CONF_IPT_SNAT_ACCESS]) != 1)
+                        return 0;
+                }
+                else
+                {
                     return 0;
+                }
             }
         }
     }
@@ -950,7 +960,7 @@ process_spa_request(const fko_srv_options_t * const opts,
     struct fw_chain * const dnat_chain = &(opts->fw_config->chain[IPT_DNAT_ACCESS]);
     struct fw_chain *snat_chain; /* We assign this later (if we need to). */
 
-    int             res = 0, is_err;
+    int             res = 0, is_err, snat_chain_num = 0;
     time_t          now;
     unsigned int    exp_ts;
 
@@ -1099,11 +1109,19 @@ process_spa_request(const fko_srv_options_t * const opts,
             if(ndx != NULL)
             {
                 strlcpy(nat_ip, spadat->nat_access, (ndx-spadat->nat_access)+1);
+                if (! is_valid_ipv4_addr(nat_ip))
+                {
+                    log_msg(LOG_INFO, "Invalid NAT IP in SPA message");
+                    free_acc_port_list(port_list);
+                    return res;
+                }
+
                 nat_port = strtol_wrapper(ndx+1, 0, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
                 if(is_err != FKO_SUCCESS)
                 {
                     log_msg(LOG_INFO, "Invalid NAT port in SPA message");
                     free_acc_port_list(port_list);
+                    res = is_err;
                     return res;
                 }
             }
@@ -1122,6 +1140,15 @@ process_spa_request(const fko_srv_options_t * const opts,
                 in_chain->target
             );
 
+            /* Check to make sure that the jump rules exist for each
+             * required chain
+            */
+            if(chain_exists(opts, IPT_INPUT_ACCESS) == 0)
+                create_chain(opts, IPT_INPUT_ACCESS);
+
+            if(jump_rule_exists(opts, IPT_INPUT_ACCESS) == 0)
+                add_jump_rule(opts, IPT_INPUT_ACCESS);
+
             if(rule_exists(opts, in_chain, rule_buf,
                         fst_proto, spadat->use_src_ip, nat_port, exp_ts) == 0)
             {
@@ -1188,7 +1215,7 @@ process_spa_request(const fko_srv_options_t * const opts,
 
         if(strlen(dnat_chain->to_chain))
         {
-            /* Make sure the required chain and jump rule exists
+            /* Make sure the required chain and jump rule exist
             */
             if(chain_exists(opts, IPT_DNAT_ACCESS) == 0)
                 create_chain(opts, IPT_DNAT_ACCESS);
@@ -1232,18 +1259,35 @@ process_spa_request(const fko_srv_options_t * const opts,
 
         /* If SNAT (or MASQUERADE) is wanted, then we add those rules here as well.
         */
-        if(strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1) == 0)
+        if(acc->force_snat || strncasecmp(opts->config[CONF_ENABLE_IPT_SNAT], "Y", 1) == 0)
         {
-            /* Setup some parameter depending on whether we are using SNAT
-             * or MASQUERADE.
+            /* Add SNAT or MASQUERADE rules.
             */
-            if(strncasecmp(opts->config[CONF_SNAT_TRANSLATE_IP], "__CHANGEME__", 10)!=0)
+            if(acc->force_snat && is_valid_ipv4_addr(acc->force_snat_ip))
+            {
+                /* Using static SNAT */
+                snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
+                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                    "--to-source %s:%i", acc->force_snat_ip, fst_port);
+                snat_chain_num = IPT_SNAT_ACCESS;
+            }
+            else if(acc->force_snat && acc->force_masquerade)
+            {
+                /* Using MASQUERADE */
+                snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
+                snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
+                    "--to-ports %i", fst_port);
+                snat_chain_num = IPT_MASQUERADE_ACCESS;
+            }
+            else if((opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
+                && is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
             {
                 /* Using static SNAT */
                 snat_chain = &(opts->fw_config->chain[IPT_SNAT_ACCESS]);
                 snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
                     "--to-source %s:%i", opts->config[CONF_SNAT_TRANSLATE_IP],
                     fst_port);
+                snat_chain_num = IPT_SNAT_ACCESS;
             }
             else
             {
@@ -1251,8 +1295,15 @@ process_spa_request(const fko_srv_options_t * const opts,
                 snat_chain = &(opts->fw_config->chain[IPT_MASQUERADE_ACCESS]);
                 snprintf(snat_target, SNAT_TARGET_BUFSIZE-1,
                     "--to-ports %i", fst_port);
+                snat_chain_num = IPT_MASQUERADE_ACCESS;
             }
 
+            if(chain_exists(opts, snat_chain_num) == 0)
+                create_chain(opts, snat_chain_num);
+
+            if(jump_rule_exists(opts, snat_chain_num) == 0)
+                add_jump_rule(opts, snat_chain_num);
+
             memset(rule_buf, 0, CMD_BUFSIZE);
 
             snprintf(rule_buf, CMD_BUFSIZE-1, IPT_SNAT_RULE_ARGS,
@@ -1487,6 +1538,39 @@ check_firewall_rules(const fko_srv_options_t * const opts)
     }
 }
 
+int
+validate_ipt_chain_conf(const char * const chain_str)
+{
+    int         j, rv  = 1;
+    const char   *ndx  = chain_str;
+
+    j = 1;
+    while(*ndx != '\0')
+    {
+        if(*ndx == ',')
+            j++;
+
+        if(*ndx != '\0'
+                && *ndx != ' '
+                && *ndx != ','
+                && *ndx != '_'
+                && isalnum(*ndx) == 0)
+        {
+            rv = 0;
+            break;
+        }
+        ndx++;
+    }
+
+    /* Sanity check - j should be the number of chain fields
+     * (excluding the type).
+    */
+    if(j != FW_NUM_CHAIN_FIELDS)
+        rv = 0;
+
+    return rv;
+}
+
 #endif /* FIREWALL_IPTABLES */
 
 /***EOF***/
