@@ -75,16 +75,55 @@ main(int argc, char **argv)
 
             if(old_pid > 0)
             {
-                res = kill(old_pid, SIGTERM);
-                if(res == 0)
+                res    = kill(old_pid, SIGTERM);
+                is_err = kill(old_pid, 0);
+
+                if(res == 0 && is_err != 0)
                 {
                     fprintf(stdout, "Killed fwknopd (pid=%i)\n", old_pid);
                     clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
                 }
                 else
                 {
-                    perror("Unable to kill fwknop: ");
-                    clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                    /* give a bit of time for process shutdown and check again
+                    */
+                    sleep(1);
+                    is_err = kill(old_pid, 0);
+                    if(is_err != 0)
+                    {
+                        fprintf(stdout, "Killed fwknopd (pid=%i) via SIGTERM\n",
+                                old_pid);
+                        clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+                    }
+                    else
+                    {
+                        res    = kill(old_pid, SIGKILL);
+                        is_err = kill(old_pid, 0);
+                        if(res == 0 && is_err != 0)
+                        {
+                            fprintf(stdout,
+                                    "Killed fwknopd (pid=%i) via SIGKILL\n",
+                                    old_pid);
+                            clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+                        }
+                        else
+                        {
+                            sleep(1);
+                            is_err = kill(old_pid, 0);
+                            if(is_err != 0)
+                            {
+                                fprintf(stdout,
+                                        "Killed fwknopd (pid=%i) via SIGKILL\n",
+                                        old_pid);
+                                clean_exit(&opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+                            }
+                            else
+                            {
+                                perror("Unable to kill fwknop: ");
+                                clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                            }
+                        }
+                    }
                 }
             }
             else
@@ -182,7 +221,7 @@ main(int argc, char **argv)
          * of an existing fwknopd process.
         */
         if(fw_config_init(&opts) != 1)
-            clean_exit(&opts, FW_CLEANUP, EXIT_FAILURE);
+            clean_exit(&opts, NO_FW_CLEANUP, EXIT_FAILURE);
 
         if(opts.fw_list == 1 || opts.fw_list_all == 1)
         {
@@ -692,7 +731,7 @@ write_pid_file(fko_srv_options_t *opts)
 static pid_t
 get_running_pid(const fko_srv_options_t *opts)
 {
-    int     op_fd, is_err;
+    int     op_fd, is_err, bytes_read = 0;
     char    buf[PID_BUFLEN] = {0};
     pid_t   rpid            = 0;
 
@@ -712,7 +751,8 @@ get_running_pid(const fko_srv_options_t *opts)
         return(rpid);
     }
 
-    if (read(op_fd, buf, PID_BUFLEN) > 0)
+    bytes_read = read(op_fd, buf, PID_BUFLEN);
+    if (bytes_read > 0)
     {
         buf[PID_BUFLEN-1] = '\0';
         /* max pid value is configurable on Linux
@@ -722,6 +762,8 @@ get_running_pid(const fko_srv_options_t *opts)
         if(is_err != FKO_SUCCESS)
             rpid = 0;
     }
+    else if (bytes_read < 0)
+        perror("Error trying to read() PID file: ");
 
     close(op_fd);
 
