@@ -8,9 +8,9 @@
  *
  * Purpose: Header file for fwknopd server program.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -57,11 +57,6 @@
   #define   LOCK_UN        0x08      /* unlock file */
 #endif
 
-/* Used by the get_user_pw function.
-*/
-#define CRYPT_OP_ENCRYPT 1
-#define CRYPT_OP_DECRYPT 2
-
 #define PID_BUFLEN 7
 
 #endif  /* FWKNOPD_H */
