@@ -85,7 +85,7 @@
 #define DEF_INTERFACE                   "eth0"
 #define DEF_ENABLE_PCAP_PROMISC         "N"
 #define DEF_PCAP_FILTER                 "udp port 62201"
-#define DEF_PCAP_DISPATCH_COUNT         "0"
+#define DEF_PCAP_DISPATCH_COUNT         "100"
 #define DEF_PCAP_LOOP_SLEEP             "100000" /* a tenth of a second (in microseconds) */
 #define DEF_ENABLE_PCAP_ANY_DIRECTION   "N"
 #define DEF_ENABLE_SPA_PACKET_AGING     "Y"
@@ -321,10 +321,20 @@ typedef struct acc_stanza
     time_t               access_expire_time;
     int                  expired;
     int                  encryption_mode;
+
+    /* DNAT parameters
+    */
     unsigned char        force_nat;
     char                *force_nat_ip;
     char                *force_nat_proto;
     unsigned int         force_nat_port;
+
+    /* SNAT parameters
+    */
+    unsigned char        force_snat;
+    char                *force_snat_ip;
+    unsigned char        force_masquerade;
+
     struct acc_stanza   *next;
 } acc_stanza_t;
 
@@ -372,6 +382,10 @@ typedef struct acc_stanza
   struct fw_config {
       struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
       char            fw_command[MAX_PATH_LEN];
+
+      /* Flag for iptables SNAT vs. MASQUERADE usage
+      */
+      unsigned char   use_masquerade;
   };
 
 #elif FIREWALL_IPFW
@@ -492,6 +506,9 @@ typedef struct fko_srv_options
     */
     struct fw_config *fw_config;
 
+    /* Set to 1 when messages have to go through syslog, 0 otherwise */
+    unsigned char   syslog_enable;
+
 } fko_srv_options_t;
 
 extern fko_srv_options_t options;
