@@ -44,6 +44,8 @@
 #include "fwknopd_errors.h"
 #include "replay_cache.h"
 
+#define CTX_DUMP_BUFSIZE            4096                /*!< Maximum size allocated to a FKO context dump */
+
 /* Validate and in some cases preprocess/reformat the SPA data.  Return an
  * error code value if there is any indication the data is not valid spa data.
 */
@@ -273,6 +275,7 @@ incoming_spa(fko_srv_options_t *opts)
     int             added_replay_digest = 0, pkt_data_len=0;
     int             is_err, cmd_exec_success = 0, attempted_decrypt = 0;
     int             conf_pkt_age = 0;
+    char            dump_buf[CTX_DUMP_BUFSIZE];
 
     spa_pkt_info_t *spa_pkt = &(opts->spa_pkt);
 
@@ -528,8 +531,10 @@ incoming_spa(fko_srv_options_t *opts)
 
         /* Add this SPA packet into the replay detection cache
         */
-        if (added_replay_digest == 0)
+        if (added_replay_digest == 0
+                && strncasecmp(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE], "Y", 1) == 0)
         {
+
             res = add_replay(opts, raw_digest);
             if (res != SPA_MSG_SUCCESS)
             {
@@ -544,8 +549,14 @@ incoming_spa(fko_srv_options_t *opts)
         /* At this point, we assume the SPA data is valid.  Now we need to see
          * if it meets our access criteria.
         */
-        log_msg(LOG_DEBUG, "[%s] (stanza #%d) SPA Decode (res=%i):\n%s",
-            spadat.pkt_source_ip, stanza_num, res, dump_ctx(ctx));
+        log_msg(LOG_DEBUG, "[%s] (stanza #%d) SPA Decode (res=%i):",
+            spadat.pkt_source_ip, stanza_num, res);
+
+        res = dump_ctx_to_buffer(ctx, dump_buf, sizeof(dump_buf));
+        if (res == FKO_SUCCESS)
+            log_msg(LOG_DEBUG, "%s", dump_buf);
+        else
+            log_msg(LOG_WARNING, "Unable to dump FKO context: %s", fko_errstr(res));
 
         /* First, if this is a GPG message, and GPG_REMOTE_ID list is not empty,
          * then we need to make sure this incoming message is signer ID matches
@@ -632,11 +643,28 @@ incoming_spa(fko_srv_options_t *opts)
             continue;
         }
 
+        if((spa_ip_demark-spadat.spa_message) < MIN_IPV4_STR_LEN-1
+                || (spa_ip_demark-spadat.spa_message) > MAX_IPV4_STR_LEN)
+        {
+            log_msg(LOG_WARNING, "[%s] (stanza #%d) Invalid source IP in SPA message, ignoring SPA packet",
+                spadat.pkt_source_ip, stanza_num, fko_errstr(res));
+
+            if(ctx != NULL)
+            {
+                if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
+                    log_msg(LOG_WARNING,
+                        "[%s] (stanza #%d) fko_destroy() could not zero out sensitive data buffer.",
+                        spadat.pkt_source_ip, stanza_num, fko_errstr(res)
+                    );
+                ctx = NULL;
+            }
+            break;
+        }
+
         strlcpy(spadat.spa_message_src_ip,
             spadat.spa_message, (spa_ip_demark-spadat.spa_message)+1);
 
-        if(strnlen(spadat.spa_message_src_ip,
-                MIN_IPV4_STR_LEN) < MIN_IPV4_STR_LEN)
+        if(! is_valid_ipv4_addr(spadat.spa_message_src_ip))
         {
             log_msg(LOG_WARNING, "[%s] (stanza #%d) Invalid source IP in SPA message, ignoring SPA packet",
                 spadat.pkt_source_ip, stanza_num, fko_errstr(res));
