@@ -11,9 +11,9 @@
  *          (ndbm or gdbm in ndbm compatibility mode) file to store the digest
  *          of a previously received SPA packets.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -109,11 +109,14 @@ rotate_digest_cache_file(fko_srv_options_t *opts)
 #if USE_FILE_CACHE
     strlcpy(new_file, opts->config[CONF_DIGEST_FILE],
         strlen(opts->config[CONF_DIGEST_FILE])+5);
+    strlcat(new_file, "-old",
+            strlen(opts->config[CONF_DIGEST_FILE])+5);
 #else
     strlcpy(new_file, opts->config[CONF_DIGEST_DB_FILE],
         strlen(opts->config[CONF_DIGEST_DB_FILE])+5);
+    strlcat(new_file, "-old",
+            strlen(opts->config[CONF_DIGEST_DB_FILE])+5);
 #endif
-    strcat(new_file, "-old");
 
 #if USE_FILE_CACHE
     res = rename(opts->config[CONF_DIGEST_FILE], new_file);
@@ -130,6 +133,9 @@ rotate_digest_cache_file(fko_srv_options_t *opts)
 #endif
         );
 #endif /* NO_DIGEST_CACHE */
+
+    if(new_file != NULL)
+        free(new_file);
 }
 
 static void
@@ -137,10 +143,10 @@ replay_warning(fko_srv_options_t *opts, digest_cache_info_t *digest_info)
 {
     char        src_ip[INET_ADDRSTRLEN+1] = {0};
     char        orig_src_ip[INET_ADDRSTRLEN+1] = {0};
-    char        created[DATE_LEN];
+    char        created[DATE_LEN] = {0};
 
 #if ! USE_FILE_CACHE
-    char        first[DATE_LEN], last[DATE_LEN];
+    char        first[DATE_LEN] = {0}, last[DATE_LEN] = {0};
 #endif
 
     /* Convert the IPs to a human readable form
@@ -201,34 +207,13 @@ replay_warning(fko_srv_options_t *opts, digest_cache_info_t *digest_info)
     return;
 }
 
-int
-replay_cache_init(fko_srv_options_t *opts)
-{
-#ifdef NO_DIGEST_CACHE
-    return(-1);
-#else
-
-    /* If rotation was specified, do it.
-    */
-    if(opts->rotate_digest_cache)
-        rotate_digest_cache_file(opts);
-
 #if USE_FILE_CACHE
-    return replay_file_cache_init(opts);
-#else
-    return replay_db_cache_init(opts);
-#endif
-
-#endif /* NO_DIGEST_CACHE */
-}
-
-#if USE_FILE_CACHE
-int
+static int
 replay_file_cache_init(fko_srv_options_t *opts)
 {
     FILE           *digest_file_ptr = NULL;
     unsigned int    num_lines = 0, digest_ctr = 0;
-    char            line_buf[MAX_LINE_LEN] = {0};
+    char            line_buf[MAX_LINE_LEN]    = {0};
     char            src_ip[INET_ADDRSTRLEN+1] = {0};
     char            dst_ip[INET_ADDRSTRLEN+1] = {0};
     long int        time_tmp;
@@ -256,7 +241,9 @@ replay_file_cache_init(fko_srv_options_t *opts)
         /* the file does not exist yet, so it will be created when the first
          * successful SPA packet digest is written to disk
         */
-        digest_file_fd = open(opts->config[CONF_DIGEST_FILE], O_WRONLY|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+        digest_file_fd = open(opts->config[CONF_DIGEST_FILE],
+                O_WRONLY|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR);
+
         if (digest_file_fd == -1)
         {
             log_msg(LOG_WARNING, "Could not create digest cache: %s: %s",
@@ -268,7 +255,7 @@ replay_file_cache_init(fko_srv_options_t *opts)
             if(write(digest_file_fd, digest_header, strlen(digest_header))
                     != strlen(digest_header)) {
                 log_msg(LOG_WARNING,
-                    "Did not write expected number of bytes to digest cache: %s\n",
+                    "Did not write expected number of bytes to digest cache: %s",
                     opts->config[CONF_DIGEST_FILE]);
             }
             close(digest_file_fd);
@@ -277,7 +264,8 @@ replay_file_cache_init(fko_srv_options_t *opts)
         }
     }
 
-    verify_file_perms_ownership(opts->config[CONF_DIGEST_FILE]);
+    if(verify_file_perms_ownership(opts->config[CONF_DIGEST_FILE]) != 1)
+        return(-1);
 
     /* File exists, and we have access - create in-memory digest cache
     */
@@ -306,13 +294,13 @@ replay_file_cache_init(fko_srv_options_t *opts)
         */
         if ((digest_elm = calloc(1, sizeof(struct digest_cache_list))) == NULL)
         {
-            fprintf(stderr, "Could not allocate digest list element\n");
+            log_msg(LOG_ERR, "[*] Could not allocate digest list element");
             continue;
         }
         if ((digest_elm->cache_info.digest = calloc(1, MAX_DIGEST_SIZE+1)) == NULL)
         {
             free(digest_elm);
-            fprintf(stderr, "Could not allocate digest string\n");
+            log_msg(LOG_ERR, "[*] Could not allocate digest string");
             continue;
         }
         src_ip[0] = '\0';
@@ -327,11 +315,10 @@ replay_file_cache_init(fko_srv_options_t *opts)
             &(digest_elm->cache_info.dst_port),
             &time_tmp) != 7)
         {
-            if(opts->verbose)
-                fprintf(stderr,
-                    "*Skipping invalid digest file entry in %s at line %i.\n - %s",
-                    opts->config[CONF_DIGEST_FILE], num_lines, line_buf
-                );
+            log_msg(LOG_INFO,
+                "*Skipping invalid digest file entry in %s at line %i.\n - %s",
+                opts->config[CONF_DIGEST_FILE], num_lines, line_buf
+            );
             free(digest_elm->cache_info.digest);
             free(digest_elm);
             continue;
@@ -358,7 +345,7 @@ replay_file_cache_init(fko_srv_options_t *opts)
         digest_ctr++;
 
         if(opts->verbose > 3)
-            fprintf(stderr,
+            log_msg(LOG_DEBUG,
                 "DIGEST FILE: %s, VALID LINE: %s",
                 opts->config[CONF_DIGEST_FILE], line_buf
             );
@@ -375,7 +362,7 @@ replay_file_cache_init(fko_srv_options_t *opts)
 /* Check for the existence of the replay dbm file, and create it if it does
  * not exist.  Returns the number of db entries or -1 on error.
 */
-int
+static int
 replay_db_cache_init(fko_srv_options_t *opts)
 {
 #ifdef NO_DIGEST_CACHE
@@ -435,41 +422,8 @@ replay_db_cache_init(fko_srv_options_t *opts)
 }
 #endif /* USE_FILE_CACHE */
 
-/* Take an fko context, pull the digest and use it as the key to check the
- * replay db (digest cache).
-*/
-int
-is_replay(fko_srv_options_t *opts, char *digest)
-{
-#ifdef NO_DIGEST_CACHE
-    return(-1);
-#else
-
 #if USE_FILE_CACHE
-    return is_replay_file_cache(opts, digest);
-#else
-    return is_replay_dbm_cache(opts, digest);
-#endif
-#endif /* NO_DIGEST_CACHE */
-}
-
-int
-add_replay(fko_srv_options_t *opts, char *digest)
-{
-#ifdef NO_DIGEST_CACHE
-    return(-1);
-#else
-
-#if USE_FILE_CACHE
-    return add_replay_file_cache(opts, digest);
-#else
-    return add_replay_dbm_cache(opts, digest);
-#endif
-#endif /* NO_DIGEST_CACHE */
-}
-
-#if USE_FILE_CACHE
-int
+static int
 is_replay_file_cache(fko_srv_options_t *opts, char *digest)
 {
     int         digest_len = 0;
@@ -484,7 +438,8 @@ is_replay_file_cache(fko_srv_options_t *opts, char *digest)
             digest_list_ptr != NULL;
             digest_list_ptr = digest_list_ptr->next) {
 
-        if (strncmp(digest_list_ptr->cache_info.digest, digest, digest_len) == 0) {
+        if (constant_runtime_cmp(digest_list_ptr->cache_info.digest,
+                    digest, digest_len) == 0) {
 
             replay_warning(opts, &(digest_list_ptr->cache_info));
 
@@ -494,7 +449,7 @@ is_replay_file_cache(fko_srv_options_t *opts, char *digest)
     return(SPA_MSG_SUCCESS);
 }
 
-int
+static int
 add_replay_file_cache(fko_srv_options_t *opts, char *digest)
 {
     FILE       *digest_file_ptr = NULL;
@@ -563,7 +518,7 @@ add_replay_file_cache(fko_srv_options_t *opts, char *digest)
 #endif /* USE_FILE_CACHE */
 
 #if !USE_FILE_CACHE
-int
+static int
 is_replay_dbm_cache(fko_srv_options_t *opts, char *digest)
 {
 #ifdef NO_DIGEST_CACHE
@@ -633,7 +588,7 @@ is_replay_dbm_cache(fko_srv_options_t *opts, char *digest)
 #endif /* NO_DIGEST_CACHE */
 }
 
-int
+static int
 add_replay_dbm_cache(fko_srv_options_t *opts, char *digest)
 {
 #ifdef NO_DIGEST_CACHE
@@ -747,5 +702,64 @@ free_replay_list(fko_srv_options_t *opts)
 }
 #endif
 
+int
+replay_cache_init(fko_srv_options_t *opts)
+{
+#ifdef NO_DIGEST_CACHE
+    return(-1);
+#else
+
+    /* If rotation was specified, do it.
+    */
+    if(opts->rotate_digest_cache)
+        rotate_digest_cache_file(opts);
+
+#if USE_FILE_CACHE
+    return replay_file_cache_init(opts);
+#else
+    return replay_db_cache_init(opts);
+#endif
+
+#endif /* NO_DIGEST_CACHE */
+}
+
+int
+add_replay(fko_srv_options_t *opts, char *digest)
+{
+#ifdef NO_DIGEST_CACHE
+    return(-1);
+#else
+
+    if(digest == NULL)
+    {
+        log_msg(LOG_WARNING, "NULL digest passed into add_replay()");
+        return(SPA_MSG_DIGEST_CACHE_ERROR);
+    }
+
+#if USE_FILE_CACHE
+    return add_replay_file_cache(opts, digest);
+#else
+    return add_replay_dbm_cache(opts, digest);
+#endif
+#endif /* NO_DIGEST_CACHE */
+}
+
+/* Take an fko context, pull the digest and use it as the key to check the
+ * replay db (digest cache).
+*/
+int
+is_replay(fko_srv_options_t *opts, char *digest)
+{
+#ifdef NO_DIGEST_CACHE
+    return(-1);
+#else
+
+#if USE_FILE_CACHE
+    return is_replay_file_cache(opts, digest);
+#else
+    return is_replay_dbm_cache(opts, digest);
+#endif
+#endif /* NO_DIGEST_CACHE */
+}
 
 /***EOF***/
