@@ -62,14 +62,7 @@ int replay_cache_init(fko_srv_options_t *opts);
 int is_replay(fko_srv_options_t *opts, char *digest);
 int add_replay(fko_srv_options_t *opts, char *digest);
 #ifdef USE_FILE_CACHE
-int replay_file_cache_init(fko_srv_options_t *opts);
-int is_replay_file_cache(fko_srv_options_t *opts, char *digest);
-int add_replay_file_cache(fko_srv_options_t *opts, char *digest);
 void free_replay_list(fko_srv_options_t *opts);
-#else
-int replay_db_cache_init(fko_srv_options_t *opts);
-int is_replay_dbm_cache(fko_srv_options_t *opts, char *digest);
-int add_replay_dbm_cache(fko_srv_options_t *opts, char *digest);
 #endif
 
 #endif  /* REPLAY_CACHE_H */
