@@ -8,9 +8,9 @@
  * Purpose: Spawns off a dummy tcp server for fwknopd.  Its purpose is
  *          to accept a tcp connection, then drop it after the first packet.
  *
- * Copyright 2010 Damien Stuart (dstuart@dstuart.org)
+ * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
  *
- *  License (GNU Public License):
+ *  License (GNU General Public License):
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -56,14 +56,21 @@ run_tcp_server(fko_srv_options_t *opts)
 {
     pid_t               pid, ppid;
     int                 s_sock, c_sock, sfd_flags, clen, selval;
-    int                 reuse_addr = 1;
+    int                 reuse_addr = 1, is_err;
     fd_set              sfd_set;
     struct sockaddr_in  saddr, caddr;
     struct timeval      tv;
-    char                sipbuf[MAX_IPV4_STR_LEN];
+    char                sipbuf[MAX_IPV4_STR_LEN] = {0};
 
-    unsigned short      port = atoi(opts->config[CONF_TCPSERV_PORT]);
+    unsigned short      port;
 
+    port = strtol_wrapper(opts->config[CONF_TCPSERV_PORT],
+            0, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "[*] Invalid max TCPSERV_PORT value.");
+        exit(EXIT_FAILURE);
+    }
     log_msg(LOG_INFO, "Kicking off TCP server to listen on port %i.", port);
 
     /* Fork off a child process to run the command and provide its outputs.
@@ -101,7 +108,12 @@ run_tcp_server(fko_srv_options_t *opts)
 
     /* So that we can re-bind to it without TIME_WAIT problems
     */
-    setsockopt(s_sock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof(reuse_addr));
+    if(setsockopt(s_sock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof(reuse_addr)) == -1)
+    {
+        log_msg(LOG_ERR, "run_tcp_server: setsockopt error: %s",
+            strerror(errno));
+        exit(EXIT_FAILURE);
+    }
 
     /* Make our main socket non-blocking so we don't have to be stuck on
      * listening for incoming connections.
